/**
 *
 *   Excalibur - A KNX/TP1 FlexNode
 *
 *   CS8900 Test Program
 *
 *   remove USB JP3 and INT JP11
 *   remove JP2 TPUART1 and JP14 TPUART2
 *   set JP11 to 3
 *
 *   Fritz Praus
 *   TU Wien
 *   fritz@praus.at
 *
 **/


#include "mb90330.h"
#include "config.h"
#include "uartapi.h"
#include "ip.h"
#include "buffers.h"
#include "tcp.h"
#include "cs8900.h"
#include "helper.h"
#include "rtc.h"

TCPCON con;

extern IPRCVle * ip_read;
extern IPRCVle * ip_write;

void showMenu(void);


const byte testip[]= {192, 168, 1, 100};
const byte testhw[]={0x00, 0x02, 0x03, 0x04, 0x05, 0x06};


const byte testip1[]= {192, 168, 1, 101};
const byte testhw1[]={0x01, 0x02, 0x03, 0x04, 0x05, 0x06};


const byte testip2[]= {192, 168, 1, 102};
const byte testhw2[]={0x02, 0x02, 0x03, 0x04, 0x05, 0x06};

void main(void)
{
	word test;
	
	/**************** generic stuff ****************/
	// configuration setup
	//initBoard();
	//initP6X();
	initUART0();
	initUART1();
	//initKNX0();
	//initKNX1();
	//initSD();
	initETH10();
	//initUSBHost();
	//initUSBSlave();


	// init UART_DEBUG
	initUART(UART_DEBUG,19200,8,0,1);  // for debug messages
	initUARTir(UART_DEBUG);

	// enable interrupts
	InitIrqLevels();
	__set_il(7);                /* allow all levels          */
	__EI();                     /* globaly enable interrupts */



	/**************** start cs8900 test program ****************/
	initRTC();
	initBuffers();
	
	/** test**/
	wait(1000);
	inportb(0x001);  // switch to 16 bit mode
	inportb(0x000);
	inportb(0x001);

/*	test = *(volatile word __far*)(0xFFFFL + IO_BASE + PP_DATA);
	printfUART(UART_DEBUG,"1: %d\r\n",test);

	test = *(volatile word __far*)(0xFFFF00L + IO_BASE + PP_DATA);
	printfUART(UART_DEBUG,"2: %d\r\n",test);

	test = *(volatile word __far*)(0x6100L + IO_BASE + PP_DATA);
	printfUART(UART_DEBUG,"3: %d\r\n",test);

	test = *(volatile word __far*)(0x610000L + IO_BASE + PP_DATA);
	printfUART(UART_DEBUG,"4: %d\r\n",test);

	test = *(volatile word __far*)(0x7100L + IO_BASE + PP_DATA);
	printfUART(UART_DEBUG,"5: %d\r\n",test);

	test = *(volatile word __far*)(0x710000L + IO_BASE + PP_DATA);
	printfUART(UART_DEBUG,"6: %d\r\n",test);

	test = *(volatile word __far*)(0x4100L + IO_BASE + PP_DATA);
	printfUART(UART_DEBUG,"7: %d\r\n",test);

	test = *(volatile word __far*)(0x710000L + IO_BASE + PP_DATA);
	printfUART(UART_DEBUG,"8: %d\r\n",test);


	test = *(volatile word __far*)(0x10000L + IO_BASE + PP_DATA);
	printfUART(UART_DEBUG,"9: %d\r\n",test);

	test = *(volatile word __far*)(0x1000000L + IO_BASE + PP_DATA);
	printfUART(UART_DEBUG,"10: %d\r\n",test);

  *(volatile word __far*)(0x10000L + IO_BASE + PP_POINTER) = 0x0102;
	test = *(volatile word __far*)(0x10000L + IO_BASE + PP_DATA);
	printfUART(UART_DEBUG,"return 9: %d\r\n",test);
*/

	printfUART(UART_DEBUG,"starting test...\r\n");
	test = inport(IO_BASE+PP_DATA);
	printfUART(UART_DEBUG,"packet page data port:0x%d\r\n",test);
	outport((IO_BASE + PP_POINTER), 0x0102);
	test = inport(IO_BASE+PP_DATA);
	printfUART(UART_DEBUG,"packet page data port:0x%d\r\n",test);
	
	initCS8900();
	
/*	printfUART(UART_DEBUG,"starting test...\r\n");
	test = inport(IO_BASE+PP_DATA);
	printfUART(UART_DEBUG,"packet page data port:0x%d\r\n",test);
	outport((IO_BASE + PP_POINTER), 0x0102);
	//outport((IO_BASE + PP_POINTER), 0x158);
	test = inport(IO_BASE+PP_DATA);
	printfUART(UART_DEBUG,"packet page data port:0x%d\r\n",test);*/

	
	
	con.tcpstate=LISTEN;
	showMenu();
	while (1) {

	    while(ip_read==ip_write); // wait until telegram received

		#ifdef DEBUG_ENABLE_ALL
			printfUART(UART_DEBUG,"Data received via CS8900\r\n");
		#endif
    
    	#ifdef LAN_SEND
			arpmanager(ip_read->data,ip_read->length);
		#endif
		#ifdef SLIP_SEND
			ipmanager(ip_read->data,ip_read->length);
		#endif
		
		ip_read=ip_read->next;

	} // while 1
}



/**
 * ISR for UART1
 */
__interrupt void rcUART1(void) { 
	byte key=0;

	if (SSR1_RDRF==1) { // receive complete UART1
		//error checking
		if (SSR1_ORE | SSR1_FRE | SSR1_PE) {     	/* error */
			key=0xFF;
		}
		else {
			key=SIDR1;
		}
		
		// handle user input
		switch (key) {
			case 'm':
				showMenu();
				break;
			case 'a':
				manageARPentry(testip, testhw);
				break;
			case 's':
				manageARPentry(testip1, testhw1);
				break;
			case 'd':
				manageARPentry(testip2, testhw2);
				break;
		}
	}
}


void showMenu(void) {
	printfUART(UART_DEBUG," _____  __    __  _____       ___   _       _   _____   _   _   _____   \r\n");
	printfUART(UART_DEBUG,"| ____| \\ \\  / / /  ___|     /   | | |     | | |  _  \\ | | | | |  _  \\  \r\n");
	printfUART(UART_DEBUG,"| |__    \\ \\/ /  | |        / /| | | |     | | | |_| | | | | | | |_| |  \r\n");
	printfUART(UART_DEBUG,"|  __|    }  {   | |       / / | | | |     | | |  _  { | | | | |  _  /  \r\n");
	printfUART(UART_DEBUG,"| |___   / /\\ \\  | |___   / /  | | | |___  | | | |_| | | |_| | | | \\ \\  \r\n");
	printfUART(UART_DEBUG,"|_____| /_/  \\_\\ \\_____| /_/   |_| |_____| |_| |_____/ \\_____/ |_|  \\_\\ \r\n\r\n");
	printfUART(UART_DEBUG,"                C S 8 9 0 0     T E S T     T O O L\r\n\r\n\r\n");
	printfUART(UART_DEBUG,"UART0 and UART%i obviously working ;-)\r\n\r\n", UART_DEBUG);
	printfUART(UART_DEBUG,"(m)enu\r\n");
}