/* THIS SAMPLE CODE IS PROVIDED AS IS AND IS SUBJECT TO ALTERATIONS. FUJITSU */
/* MICROELECTRONICS ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR */
/* ELIGIBILITY FOR ANY PURPOSES.                                             */
/*                 (C) Fujitsu Microelectronics Europe GmbH                  */
/*---------------------------------------------------------------------------
  VECTORS.C
  - Interrupt level (priority) setting
  - Interrupt vector definition
-----------------------------------------------------------------------------*/

#include "mb90330.h"

/*---------------------------------------------------------------------------
   InitIrqLevels()
   This function  pre-sets all interrupt control registers. It can be used
   to set all interrupt priorities in static applications. If this file
   contains assignments to dedicated resources, verify  that the
   appropriate controller is used.
   NOTE: value 7 disables the interrupt and value 0 sets highest priority.
   NOTE: Two resource interrupts always share one ICR register.
-----------------------------------------------------------------------------*/

void InitIrqLevels(void)
{
/*  ICRxx               shared IRQs for ICR */

    ICR00 = 7;      /*  IRQ11
                        IRQ12 */
    ICR01 = 7;      /*  IRQ13     
                        IRQ14 */
    ICR02 = 7;      /*  IRQ15
                        IRQ16 */
    ICR03 = 7;      /*  IRQ17
                        IRQ18 */
    ICR04 = 5;      /*  IRQ19
                        IRQ20 */
    ICR05 = 7;      /*  IRQ21
                        IRQ22 */
    ICR06 = 7;      /*  IRQ23
                        IRQ24 */
    ICR07 = 7;      /*  IRQ25
                        IRQ26 */
    ICR08 = 7;      /*  IRQ27
                        IRQ28 */
    ICR09 = 7;      /*  IRQ29
                        IRQ30 */
    ICR10 = 7;      /*  IRQ31
                        IRQ32 */
    ICR11 = 7;      /*  IRQ33
                        IRQ34 */
    ICR12 = 7;      /*  IRQ35
                        IRQ36 */
    ICR13 = 7;      /*  IRQ37
                        IRQ38 */
    ICR14 = 6;      /*  IRQ39
                        IRQ40 */
    ICR15 = 7;      /*  IRQ41
                        IRQ42 */
}

/*---------------------------------------------------------------------------
   Prototypes
   Add your own prototypes here. Each vector definition needs is proto-
   type. Either do it here or include a header file containing them.
-----------------------------------------------------------------------------*/

__interrupt void DefaultIRQHandler (void);
__interrupt void rcUART1(void);
__interrupt void intCS8900(void);
__interrupt void intTimer(void);

/*---------------------------------------------------------------------------
   Vector definiton
   Use following statements to define vectors. All resource related
   vectors are predefined. Remaining software interrupts can be added here
   as well.
   NOTE: If software interrupts 0 to 7 are defined here, this might 
   conflict with the reset vector in the start-up file.
-----------------------------------------------------------------------------*/

#pragma intvect DefaultIRQHandler  9    /* software interrupt 9         */
#pragma intvect DefaultIRQHandler 10    /* exeception handler           */
#pragma intvect DefaultIRQHandler 11    /* USB function 1               */
#pragma intvect DefaultIRQHandler 12    /* USB function 2               */
#pragma intvect DefaultIRQHandler 13    /* USB function 3               */
#pragma intvect DefaultIRQHandler 14    /* USB function 4               */
#pragma intvect DefaultIRQHandler 15    /* USB Mini HOST 1              */
#pragma intvect DefaultIRQHandler 16    /* USB Mini HOST 2              */
#pragma intvect DefaultIRQHandler 17    /* I2C 0                        */
#pragma intvect DefaultIRQHandler 18    /* DTP/External Interrupt 0/1   */
#pragma intvect DefaultIRQHandler 19    /* I2C 1                        */
#pragma intvect intCS8900		  20   	 /* DTP/External Interrupt 2/3   */
#pragma intvect DefaultIRQHandler 21    /* I2C 2                        */
#pragma intvect DefaultIRQHandler 22    /* DTP/External Interrupt 4/5   */
#pragma intvect DefaultIRQHandler 23    /* PWC/Reload Timer 0           */
#pragma intvect DefaultIRQHandler 24    /* DTP/External Interrupt 6/7   */
#pragma intvect DefaultIRQHandler 25    /* Input Capture 0/1            */
#pragma intvect DefaultIRQHandler 26    /* Reload Timer 1               */
#pragma intvect DefaultIRQHandler 27    /* Input Capture 2/3            */
#pragma intvect DefaultIRQHandler 28    /* Reload Timer 2               */
#pragma intvect DefaultIRQHandler 29    /* Output Compare 0/1           */
#pragma intvect DefaultIRQHandler 30    /* PPG 0/1                      */
#pragma intvect DefaultIRQHandler 31    /* Output Compare 2/3           */
#pragma intvect DefaultIRQHandler 32    /* PPG 2/3                      */
#pragma intvect DefaultIRQHandler 33    /* UART 2 TX / UART 3 TX        */
#pragma intvect DefaultIRQHandler 34    /* PPG 4/5                      */
#pragma intvect DefaultIRQHandler 35    /* UART 2 RX / UART 3 RX        */
#pragma intvect DefaultIRQHandler 36    /* ADC / Free-run Timer         */
#pragma intvect DefaultIRQHandler 37    /* UART 0 RX / UART 1 RX TX??   */
#pragma intvect DefaultIRQHandler 38    /* Ext. SIO                     */
#pragma intvect rcUART1           39    /* UART 0 RX / UART 1 RX        */
#pragma intvect intTimer			 40    /* Timebase Timer / Watch Timer */
#pragma intvect DefaultIRQHandler 41    /* Flash Memory                 */
#pragma intvect DefaultIRQHandler 42    /* Delayed Interrupts           */

/*---------------------------------------------------------------------------
   DefaultIRQHandler()
   This function is a placeholder for all vector definitions. Either use
   your own placeholder or add necessary code here. 
-----------------------------------------------------------------------------*/

__interrupt 
void DefaultIRQHandler (void)
{
    __DI();                              /* disable interrupts */
    while(1)
        __wait_nop();                    /* halt system */
}
