/**
 *
 *   Excalibur - A KNX/TP1 FlexNode
 *
 *   SD/MMC Card Test Program
 *
 *   remove USB JP3 and INT JP11
 *   remove JP2 TPUART1 and JP14 TPUART2
 *
 *   Fritz Praus
 *   TU Wien
 *   fritz@praus.at
 *
 **/


#include "mb90330.h"
#include "config.h"
#include "uartapi.h"
#include "mmc.h"
#include "fat16.h"

void showMenu(void);
void printRootDirectory(void);
void printIndexhtml(void);
void readCID(void);
void readCSD(void);
void readSector(int address);
void writeSector(byte mode);
void append(void);
void printTesttxt(void);

void main(void)
{
	/**************** generic stuff ****************/
	// configuration setup
	initBoard();
	//initP6X();
	initUART0();
	initUART1();
	//initKNX0();
	//initKNX1();
	initSD();
	//initETH10();
	//initUSBHost();
	//initUSBSlave();


	// init UART_DEBUG
	initUART(UART_DEBUG,19200,8,0,1);  // for debug messages
	initUARTir(UART_DEBUG);

	// enable interrupts
	InitIrqLevels();
	__set_il(7);                /* allow all levels          */
	__EI();                     /* globaly enable interrupts */



	/**************** start sd/mmc test program ****************/
	showMenu();
	while (1) {
		nop();
	}
}

/**
 * ISR for UART1
 */
__interrupt void rcUART1(void) { 
	byte key=0;

	if (SSR1_RDRF==1) { // receive complete UART1
		//error checking
		if (SSR1_ORE | SSR1_FRE | SSR1_PE) {     	/* error */
			key=0xFF;
		}
		else {
			key=SIDR1;
		}
		
		// handle user input
		switch (key) {
			case 'm':
				showMenu();
				break;
			case 'i': // init sd card
				while (mmc_init() != 0) { 	//ist der Rckgabewert ungleich NULL ist ein Fehler aufgetreten
													//MMC/SD-Karte wird dann neu Initialisiert
				   printfUART(UART_DEBUG,"** Keine MMC/SD-Karte gefunden! **\r\n"); 
				}

				printfUART(UART_DEBUG,"MMC/SD-card found!\r\n"); 
				//Initialisierung der MMC/SD-Karte ENDE!
				break;
			case 'f':
				fatInit();
				break;
/* 		case 'p': // print root directory
				printRootDirectory();
				break;*/
			case 'h': // print index.html
				printIndexhtml();
				break;
			case 'c':
				printfUART(UART_DEBUG,"card inserted pin:0x%x\r\n",MMC_GetCI());
				break;
			case 'w':
				printfUART(UART_DEBUG,"card write protected pin:0x%x\r\n",MMC_GetWP());
				break;
			case 'd':
				readCID();
				break;
			case 's':
				readCSD();
				break;
			case 'r':
				readSector(10);
				break;
			case 't':
				writeSector(0);
				break;
			case 'e':
				writeSector(1);
				break;
			case 'z':
				writeSector(2);
				break;
			case 'a':
				append();
				break;
			case 'q':
				readSector(150);
				break;
/*			case 'n':
				printTesttxt();
				break;*/
			case '0':
			case '1':
			case '2':
			case '3':
			case '4':
			case '5':
			case '6':
			case '7':
			case '8':
			case '9':
				readSector(key-48);
				break;
		}
	}
}

void readSector(int add) {
	byte data[512];
	int a;
	
	mmc_read_sector (add,&data[0]);

	printfUART(UART_DEBUG,"Data at address %i: 0x", add);
	for (a=0;a<512;a++) {
		sendUARThex(UART_DEBUG,data[a]);
	}
	printfUART(UART_DEBUG,"\r\n");

	for (a=0;a<512;a++) {
		sendUARTbyte(UART_DEBUG,data[a]);
	}
	printfUART(UART_DEBUG,"\r\n");

}


void writeSector(byte mode) {
	byte data[512];
	int a;
	
	for (a=0;a<512;a++) {
		if (mode==0) {
			data[a]=a%8;
		}
		else if (mode==1) {
			data[a]=0xFF;
		}
		else {
			data[a]=0x00;
		}
	}

	mmc_write_sector (10,&data[0]);

	printfUART(UART_DEBUG,"wrote 512 bytes to address 10\r\n");
}


void printRootDirectory(void) {
/*	unsigned char Buffer[512];
	unsigned short int tmp;

	//Ausgabe des Root Directory
	unsigned int Cluster;
	unsigned char Dir_Attrib = 0;
	unsigned long Size = 0;
	char a;

	printfUART(UART_DEBUG, "Directory:\r\n");
	for (a = 0;a < 240;a++) {
		Cluster = Read_Dir_Ent(0,a,&Size,&Dir_Attrib,&Buffer[0]);
		if (Cluster == 0xffff) {
			break;
		}
		tmp = (Size & 0x0000FFFF);
		printfUART(UART_DEBUG,"Cluster = %x DirA = %x Size= %d FileName = ",Cluster,Dir_Attrib,tmp);
		printfUART(UART_DEBUG,"%s", Buffer);
		printfUART(UART_DEBUG,"\n\r");
	}
	printfUART(UART_DEBUG,"Directory Ende\r\n");*/
}

void printIndexhtml(void) {
/*	unsigned char Buffer[512];

	unsigned int Cluster;
	unsigned char Dir_Attrib = 0;
	unsigned long Size = 0;
	int a=0;
	int b=0;
	unsigned int tmp;
	
	//Lade Cluster fr das index.htm File in den Speicher 
	Cluster = 0;//suche im Root Verzeichnis
	if (Search_File((unsigned char*)"index.htm",&Cluster,&Size,&Dir_Attrib,Buffer) == 1) {

		printfUART(UART_DEBUG, "File Found!!\r\n");

		// Berechnen der Blockanzahl: 
		tmp = (Size>>9) +1;
	
		//Lese File und gebe es auf der seriellen Schnittstelle aus
		for (b = 0;b<tmp;b++) {
		
			Read_File (Cluster,Buffer,b);
			
			for (a = 0;a<512;a++) {
				printfUART(UART_DEBUG, "%c",Buffer[a]);
			}
		}
	}
	printfUART(UART_DEBUG, "FERTIG!!\r\n");*/
	char FilePathName[64];	
	strcpy( FilePathName, "index.htm");
	FAT16_Search( FilePathName , CONTINUE );
	
}

void readCID() {
	byte data[20];
	unsigned int tmp;
	
	Read_CID_MMC(&data[0]);
	printfUART(UART_DEBUG,"CID from card:0x");
	for (tmp = 0;tmp<16;tmp++) {
		printfUART(UART_DEBUG,"%x ",data[tmp]);
	}
	printfUART(UART_DEBUG,"\r\n");
}


void readCSD() {
	unsigned char Buffer[512];
	unsigned int tmp;
	
	Read_CSD_MMC (Buffer);
	
	printfUART(UART_DEBUG,"CSD from card:0x");
	for (tmp = 0;tmp<16;tmp++) {
		printfUART(UART_DEBUG, "%x ",Buffer[tmp]);
	}
	printfUART(UART_DEBUG,"\r\n");
}

void append(void) {
/*	//Schreiben von Daten in Vorhandenes File!
	unsigned int Cluster=0;
	unsigned char Dir_Attrib = 0;
	unsigned long Size = 0;
	unsigned long FatAdress = 0; // Fester Puffer fr die FAT-Adresse eines Blockes
	int b=0;
	int a=0;
	byte Buffer[512];
	byte tmp;

	// Datei Suchen und Cluster - Nr. ermitteln
	if ((Search_File((unsigned char*)"test.txt",&Cluster,&Size,&Dir_Attrib,Buffer) == 1) ) {
		printfUART(UART_DEBUG, "File Found!! Writing ...\r\n");
		
		// Berechnen der Blockanzahl: 
		tmp = (Size >>9) +1;
		
		
		// Beschreiben mit neuen Daten, Inclusive dem letzten Cluster
		for (b = 0;b<tmp;b++) {
			// Berechne Zieladresse
			FatAdress = getFatBlockAdr(Cluster,Buffer,b);	
			
			// Flle Buffer Mit Daten
			for (a = 0;a<512;a++) {
				Buffer[a]=a%8;
			}
			
			// Schreibe Daten auf Karte
			mmc_write_sector (FatAdress,Buffer);
			
		}
		printfUART(UART_DEBUG, "FERTIG!!\r\n");
	}*/
	int i;
	char FilePathName[64];
	
	strcpy( FilePathName, "test/test.hex");
	FAT16_Search( FilePathName , OVERWRITE );
	for( i=0; i<400; i++) {
		FAT16_Write_Data( (uint32_t)i );
	}
	// data has to be finished before powering of mmc card
	// else filesize won't be saved and thus data can't 
	// be read by PC
	FAT16_Finish_Data();
}

void printTesttxt(void) {
/*	unsigned int Cluster=0;
	unsigned char Dir_Attrib = 0;
	unsigned long Size = 0;
	int b=0;
	int a=0;
	byte Buffer[512];
	byte tmp;
	
	// Lesen der ge nderten Datei!
	if ((Search_File((unsigned char *)"leer.txt",&Cluster,&Size,&Dir_Attrib,Buffer) == 1) ) {
		printfUART(UART_DEBUG, "File Found!!\r\n");
		tmp = (Size >>9) +1;
		
		
		//Lese File und gibt es auf der seriellen Schnittstelle aus
		for (b = 0;b<tmp;b++) {
			Read_File (Cluster,Buffer,b);
				
			for (a = 0;a<512;a++) {				
				printfUART(UART_DEBUG,"%c",Buffer[a]);
			}
		}
	printfUART(UART_DEBUG, "FERTIG!!\r\n");	
	}*/
}


void showMenu(void) {
	printfUART(UART_DEBUG," _____  __    __  _____       ___   _       _   _____   _   _   _____   \r\n");
	printfUART(UART_DEBUG,"| ____| \\ \\  / / /  ___|     /   | | |     | | |  _  \\ | | | | |  _  \\  \r\n");
	printfUART(UART_DEBUG,"| |__    \\ \\/ /  | |        / /| | | |     | | | |_| | | | | | | |_| |  \r\n");
	printfUART(UART_DEBUG,"|  __|    }  {   | |       / / | | | |     | | |  _  { | | | | |  _  /  \r\n");
	printfUART(UART_DEBUG,"| |___   / /\\ \\  | |___   / /  | | | |___  | | | |_| | | |_| | | | \\ \\  \r\n");
	printfUART(UART_DEBUG,"|_____| /_/  \\_\\ \\_____| /_/   |_| |_____| |_| |_____/ \\_____/ |_|  \\_\\ \r\n\r\n");
	printfUART(UART_DEBUG,"              S D / M M C   C A R D     T E S T     T O O L\r\n\r\n");
	printfUART(UART_DEBUG,"UART0 and UART%i obviously working ;-)\r\n\r\n", UART_DEBUG);
	printfUART(UART_DEBUG,"(m)enu\r\n");
	printfUART(UART_DEBUG,"(c)ard inserted  (w)rite protected\r\n\r\n");
	printfUART(UART_DEBUG,"(i)nit SD/MMX card ci(d) read  c(s)d read\r\n");
	printfUART(UART_DEBUG,"read from address x(0..9)\r\n");
	printfUART(UART_DEBUG,"address 10: (r)ead 512bytes   wri(t)e (i++)%8   writ(e) 0xFF   write (z)eros\r\n\r\n");
	printfUART(UART_DEBUG,"init (f)at search index.(h)tml (a)ppend something to test.hex\r\n");
}