/**
 *
 *   Excalibur - A KNX/TP1 FlexNode
 *
 *   TP-UART Test Program
 *
 *   remove USB JP3 and INT JP11
 *   both TP-UARTS have to be connected to eib
 *
 *   Fritz Praus
 *   TU Wien
 *   fritz@praus.at
 *
 **/


#include "mb90330.h"
#include "config.h"
#include "uartapi.h"
#include "tpuart.h"



extern byte rb[20][25];		// receive queue
extern int row_read;
extern int row_write;

EIBFRAME eib_test;

void showMenu(void);
void showTPUARTstatus(int port);

void main(void)
{
	int i=0;
	byte data=0x81;

	/**************** generic stuff ****************/
	// configuration setup
	initBoard();
	//initP6X();
	initUART0();
	initUART1();
	initKNX0();
	initKNX1();
	//initSD();
	//initETH10();
	//initUSBHost();
	//initUSBSlave();


	// init UART_DEBUG
	initUART(UART_DEBUG,19200,8,0,1);  // for debug messages
	initUARTir(UART_DEBUG);

	// init TPUARTS
	initTPUART(UART_KNX0,1);
	initTPUART(UART_KNX1,1);

	// enable interrupts
	InitIrqLevels();
	__set_il(7);                /* allow all levels          */
	__EI();                     /* globaly enable interrupts */



	/**************** start uart test program ****************/
	showMenu();
	SCR2_RXE  = 0; // receive disable
	SCR3_RXE  = 0; // receive disable
	SSR2_RIE = 0; // disable receive interrupt
	SSR3_RIE = 0; // disable receive interrupt
	
	initEIBframe(&eib_test,11,0x1204,65,1,0,&data,1);
	
	while (1) {

		if(row_write!=row_read) { // new EIB frame received
			#ifdef DEBUG_ENABLE_MOST
				printfUART(UART_DEBUG,"EIB Frame received\r\n");
			#endif

			printfUART(UART_DEBUG,"received EIB Frame 0x");
			for(i=0; i<((rb[row_read][5]&0x0F)+10); i++) {
				sendUARThex(UART_DEBUG,rb[row_read][i]);
				rb[row_read][i]=0;
				sendUARTbyte(UART_DEBUG,' ');
			}
			printfUART(UART_DEBUG,"\r\n");
			row_read=(row_read+1) % 25;

  		}
	}
}

/**
 * ISR for UART1
 */
__interrupt void rcUART1(void) { 
	byte key=0;

	if (SSR1_RDRF==1) { // receive complete UART1
		//error checking
		if (SSR1_ORE | SSR1_FRE | SSR1_PE) {     	/* error */
			key=0xFF;
		}
		else {
			key=SIDR1;
		}
		
		// handle user input
		switch (key) {
			case 'm':
				showMenu();
				break;
			case 'h': // hw reset KNX0
				resetTPUARThw(UART_KNX0);
				printfUART(UART_DEBUG,"KNX0 hw reset\r\n");
				break;
			case 's': // sw reset KNX0
				resetTPUARTsw(UART_KNX0);
				printfUART(UART_DEBUG,"KNX0 sw reset\r\n");
				break;
			case 't': // status KNX0
				showTPUARTstatus(UART_KNX0);
				break;
			case 'e': // send test KNX0
				sendEIBframe(UART_KNX0,&eib_test);
				printfUART(UART_DEBUG,"sent EIB frame via KNX0\r\n");
				break;
			case 'r': // receive enable KNX0
				SCR2_RXE  = 1; // receive enable
				SSR2_RIE = 1; // enable receive interrupt
				SCR3_RXE  = 0; // receive disable 
				printfUART(UART_DEBUG,"KNX0 receive enable\r\n");				
				break;
			case 'n': // send test analog KNX0
				printfUART(UART_DEBUG,"not implemented\r\n");				
				break;
			case 'c': // receive enable analog KNX0
				printfUART(UART_DEBUG,"not implemented\r\n");				
				break;
			case 'a': // hw reset KNX1
				resetTPUARThw(UART_KNX1);
				printfUART(UART_DEBUG,"KNX1 hw reset\r\n");
				break;
			case 'o': // sw reset KNX1
				resetTPUARTsw(UART_KNX1);
				printfUART(UART_DEBUG,"KNX1 sw reset\r\n");
				break;
			case 'u': // status KNX1
				showTPUARTstatus(UART_KNX1);
				break;
			case 'd': // send test KNX1
				sendEIBframe(UART_KNX1,&eib_test);
				printfUART(UART_DEBUG,"sent EIB frame via KNX1\r\n");
				break;
			case 'i': // receive enable KNX1
				SCR2_RXE = 0; // receive disable
				SCR3_RXE = 1; // receive enable 
				SSR3_RIE = 1; // enable receive interrupt
				printfUART(UART_DEBUG,"KNX1 receive enable\r\n");
				break;
		}
	}
}


void showMenu(void) {
	printfUART(UART_DEBUG," _____  __    __  _____       ___   _       _   _____   _   _   _____   \r\n");
	printfUART(UART_DEBUG,"| ____| \\ \\  / / /  ___|     /   | | |     | | |  _  \\ | | | | |  _  \\  \r\n");
	printfUART(UART_DEBUG,"| |__    \\ \\/ /  | |        / /| | | |     | | | |_| | | | | | | |_| |  \r\n");
	printfUART(UART_DEBUG,"|  __|    }  {   | |       / / | | | |     | | |  _  { | | | | |  _  /  \r\n");
	printfUART(UART_DEBUG,"| |___   / /\\ \\  | |___   / /  | | | |___  | | | |_| | | |_| | | | \\ \\  \r\n");
	printfUART(UART_DEBUG,"|_____| /_/  \\_\\ \\_____| /_/   |_| |_____| |_| |_____/ \\_____/ |_|  \\_\\ \r\n\r\n");
	printfUART(UART_DEBUG,"                T P - U A R T     T E S T     T O O L\r\n");
	printfUART(UART_DEBUG,"UART0 and UART%i obviously working ;-)\r\n\r\n", UART_DEBUG);
	printfUART(UART_DEBUG,"(m)enu\r\n\r\n");
	printfUART(UART_DEBUG,"KNX0: (h)ardware reset, (s)oftware reset, s(t)atus\r\n");
	printfUART(UART_DEBUG,"   normal mode: s(e)nd test frame, (r)eceive enable\r\n");
	printfUART(UART_DEBUG,"   analog mode: se(n)d test frame, re(c)eive enable\r\n\r\n");
	printfUART(UART_DEBUG,"KNX1: h(a)rdware reset, s(o)ftware reset, stat(u)s\r\n");
	printfUART(UART_DEBUG,"   normal mode: sen(d) test frame, rece(i)ve enable\r\n\r\n");
}

void showTPUARTstatus(int port) {
		if (getTPUARTSavepin(port) == 0) {
			printfUART(UART_DEBUG,"TPUART connected to UART%i ok and connected. Savepin:%x\r\n", port,getTPUARTSavepin(port));
		}
		else {
			printfUART(UART_DEBUG,"Error: TPUART connected to UART%i not connected to EIB. Savepin:%x\r\n", port,getTPUARTSavepin(port));
		}
}