/**
 * config.h
 *
 * configuration for current microcontroller and setup
 *
 * @author: fritz
 */
 
#ifndef __CONFIG__
#define __CONFIG__

#include "globtype.h"
#include "mb90330.h"

/**
 * enable debug messages
 */
//#define DEBUG_ENABLE_ALL
//#define DEBUG_ENABLE_MOST
//#define DEBUG_ENABLE_LITTLE
//#define DEBUG_ENABLE_INFO
//#define DEBUG_FAT

/**
 * configuration of UARTS
 * attention: hardware dependend functions may have to be altered
 *            eg: isr
 **/
#define	UART_DEBUG	1
#define	UART_SLIP	1
#define	UART_KNX0	2
#define	UART_KNX1	3 // busmonitor

/**
 * SD/MMC card setup
 *
 **/
#define SPI_DI				PDR3_P33 // Port Pin an dem Data Output der MMC/SD-Karte angeschlossen ist 
#define SPI_DO				PDR3_P31 // Port Pin an dem Data Input der MMC/SD-Karte angeschlossen ist
#define SPI_Clock			PDR3_P32	// Port Pin an dem die Clock der MMC/SD-Karte angeschlossen ist (clk)
#define MMC_Chip_Select	PDR3_P30	// Port Pin an dem Chip Select der MMC/SD-Karte angeschlossen ist
#define MMC_CI				PDR3_P34 // 1 wenn keine Karte eingelegt, 0 wenn Karte eingelegt
#define MMC_WP				PDR3_P35 // 1 wenn Karte nicht schreibgeschuetzt, 0 wenn Karte schreibgeschuetzt

/**
 * CS 8900 setup
 *
 **/
#define CS8900_MAC45			0x0000
#define CS8900_MAC23			0x4711
#define CS8900_MAC01			0x0815
#define CS8900_IRQ			0x0002
#define CS8900_LEVELA		ELVR_LA2
#define CS8900_LEVELB		ELVR_LB2
#define CS8900_IRQENABLE	ENIR_EN2
#define CS8900_IRQREQUEST	EIRR_ER2


/**
 * send ip packets via ...
 * only define one entry
 **/
//#define	SLIP_SEND	//ip packet sent via slip
#define	LAN_SEND		//ip packet sent via lan


/**
 * configuration EIBNET/IP side
 **/
#define	EIBNETIP_MAXCONNECTIONS	1

/**
 * configuration
 **/
#define	CONFIG_SN0			0x00;
#define	CONFIG_SN1			0x01;
#define	CONFIG_SN2			0x11;
#define	CONFIG_SN3			0x11;
#define	CONFIG_SN4			0x11;
#define	CONFIG_SN5			0x11;

#define	ADDITIONAL_INDIVIDUAL_ADDRESSES_NR 1
#define	CONFIG_KNXMEDIUM	TP0;


/**
 * setup dependend prototypes
 * adapt to current jumper configuration
 * EXTREME CARE NOT TO TOAST THE HARDWARE!!!
 *
 * f.e. Data direction registers
 *
 **/
void initBoard(void);
void initP6X(void);
void initUART0(void);
void initUART1(void);
void initKNX0(void);
void initKNX1(void);
void initSD(void);
void initETH10(void);
void initUSBHost(void);
void initUSBSlave(void);




// due to missing sbrk, no dynamic linked list is used currently
// all current knx addresses for gateway
// f.e. iack messages are generated for the listed adresses
/*struct lst_knxaddr {
	unsigned int * addr; // pointer to address;not value!!! -> after devmgmt list is still valid
	struct lst_knxaddr * next;	
};
typedef struct lst_knxaddr it_addr;
*/

extern unsigned int *lst_knxaddr[ADDITIONAL_INDIVIDUAL_ADDRESSES_NR+1];

void addknxaddress(unsigned int *address);
void remknxaddress(unsigned int *address);
int findknxaddress(unsigned int *address);

/**
 * globals
 **/
extern unsigned int PROJECT_INSTALLATION_ID;
extern unsigned int KNX_INDIVIDUAL_ADDRESSES;
extern unsigned int ADDITIONAL_INDIVIDUAL_ADDRESSES[ADDITIONAL_INDIVIDUAL_ADDRESSES_NR+1];
extern unsigned char CURRENT_IP_ASSIGNMENT_METHOD;
extern unsigned char IP_ASSIGNMENT_METHOD;
extern unsigned char IP_CAPABILITIES;
extern unsigned long CURRENT_IP_ADDRESS;
extern unsigned long CURRENT_SUBNET_MASK;
extern unsigned long CURRENT_DEFAULT_GATEWAY;
extern unsigned long IP_ADDRESS;
extern unsigned long SUBNET_MASK;
extern unsigned long DEFAULT_GATEWAY;
extern unsigned long DHCP_BOOTP_SERVER;
extern unsigned long SYSTEM_SETUP_MULTICAST_ADDRESS;
extern unsigned long ROUTING_MULTICAST_ADDRESS;
extern unsigned char TTL;
extern unsigned int EIBNETIP_DEVICE_CAPABILITIES;
extern unsigned char EIBNETIP_DEVICE_STATE;
extern unsigned char EIBNETIP_ROUTING_CAPABILITIES;
extern unsigned char PRIORITY_FIFO_ENABLED;
extern unsigned int QUEUE_OVERFLOW_TO_IP;
extern unsigned int QUEUE_OVERFLOW_TO_KNX;
extern unsigned long MSG_TRANSMIT_TO_IP;
extern unsigned long MSG_TRANSMIT_TO_KNX;
extern unsigned char	FRIENDLY_NAME[30];


#endif