/** \file
 * UART90330.c
 * Hardware dependent functions for MB90330 UART's (4 UART's)
 *
 * @version $Id: UART90330.c,v 1.7 2005/10/05 16:16:14 fritz Exp $
 * @author fritz
 */
 
#include "globtype.h"
#include "uartapi.h"
#include "mb90330.h"
#include "config.h"

/**
 * initUART() for MB90330
 * @param port number of UART you want to use (0, 1, 2,...)
 * @param baud Baudrate
 * @param bit number of Bits (7,8)
 * @param parity Parity (0: None, 1: Odd, 2: Even)
 * @param stop number of Stopbits (1, 2) 
 * @return None
 */

void initUART(int port, int baud, int bit, int parity, int stop) {

	switch(port) {
		// UART 0
		case 0:
			SMR0_SOE  = 1; // output enable 
			SMR0_SCKE = 0;
			SMR0_MD1  = 0; // Normal asynchronous mode 
			SMR0_MD0  = 0;

			SCR0_TXE  = 1; // Transmit enable 
			SCR0_RXE  = 1; // receive enable 
			SCR0_REC  = 0; // clear error flags 
			SCR0_AD   = 0; 

			switch(baud) {
				case 9600:
					UTRL0 = 0x8271;
					break;
    
				case 19200:
					UTRL0 = 0x8138;
					break;
    
				default:         // set to 19200 Baud
					UTRL0 = 0x8138;
					break;
			}
    
			if(bit==8) {  
				SCR0_CL = 1; // 8 data bits 
			}
			else {	
				SCR0_CL = 0; // 7 data bits 
			}
	
			if(stop==1) {	
				SCR0_SBL = 0; // 1 stop bit 
			}
			else {
				SCR0_SBL = 1; // 2 stop bits 
			}
  
			switch(parity) {
				case 0:
					SCR0_PEN  = 0; // parity disable 
					break;
    
				case 1: 
					SCR0_P = 1; // odd parity
					SCR0_PEN = 1; // parity enable
					break;
    
				case 2:
					SCR0_P = 0; // even parity
					SCR0_PEN = 1; // parity enable
					break;
    
				default:
					SCR0_PEN = 0; // parity disable        
			}
			break; 
	
		// UART 1
		case 1:
			SMR1_SOE  = 1; // output enable 
			SMR1_SCKE = 0;
			SMR1_MD1  = 0; // Normal asynchronous mode 
			SMR1_MD0  = 0;

			SCR1_TXE  = 1; // Transmit enable 
			SCR1_RXE  = 1; // receive enable 
			SCR1_REC  = 0; // clear error flags 
			SCR1_AD   = 0; 

			switch(baud) {
				case 9600:
					UTRL1 = 0x8271;
					break;
    
				case 19200:
					UTRL1 = 0x8138;
					break;
    
				default:         // set to 19200 Baud
					UTRL1 = 0x8138;
					break;
			}
    
			if(bit==8) {  
				SCR1_CL = 1; // 8 data bits 
			}
			else {	
				SCR1_CL = 0; // 7 data bits 
			}
	
			if(stop==1) {	
				SCR1_SBL = 0; // 1 stop bit 
			}
			else {
				SCR1_SBL = 1; // 2 stop bits 
			}
  
			switch(parity) {
				case 0:
					SCR1_PEN  = 0; // parity disable 
					break;
    
				case 1: 
					SCR1_P = 1; // odd parity
					SCR1_PEN = 1; // parity enable
					break;
    
				case 2:
					SCR1_P = 0; // even parity
					SCR1_PEN = 1; // parity enable
					break;
    
				default:
					SCR1_PEN = 0; // parity disable        
			}
			break; 
		// UART 2
		case 2:
			SMR2_SOE  = 1; // output enable 
			SMR2_SCKE = 0;
			SMR2_MD1  = 0; // Normal asynchronous mode 
			SMR2_MD0  = 0;

			SCR2_TXE  = 1; // Transmit enable 
			SCR2_RXE  = 1; // receive enable 
			SCR2_REC  = 0; // clear error flags 
			SCR2_AD   = 0; 

			switch(baud) {
				case 9600:
					UTRL2 = 0x8271;
					break;
    
				case 19200:
					UTRL2 = 0x8138;
					break;
    
				default:         // set to 19200 Baud
					UTRL2 = 0x8138;
					break;
			}
    
			if(bit==8) {  
				SCR2_CL = 1; // 8 data bits 
			}
			else {	
				SCR2_CL = 0; // 7 data bits 
			}
	
			if(stop==1) {	
				SCR2_SBL = 0; // 1 stop bit 
			}
			else {
				SCR2_SBL = 1; // 2 stop bits 
			}
  
			switch(parity) {
				case 0:
					SCR2_PEN  = 0; // parity disable 
					break;
    
				case 1: 
					SCR2_P = 1; // odd parity
					SCR2_PEN = 1; // parity enable
					break;
    
				case 2:
					SCR2_P = 0; // even parity
					SCR2_PEN = 1; // parity enable
					break;
    
				default:
					SCR2_PEN = 0; // parity disable        
			}
			break; 
		// UART 3
		case 3:
			SMR3_SOE  = 1; // output enable 
			SMR3_SCKE = 0;
			SMR3_MD1  = 0; // Normal asynchronous mode 
			SMR3_MD0  = 0;

			SCR3_TXE  = 1; // Transmit enable 
			SCR3_RXE  = 1; // receive enable 
			SCR3_REC  = 0; // clear error flags 
			SCR3_AD   = 0; 

			switch(baud) {
				case 9600:
					UTRL3 = 0x8271;
					break;
    
				case 19200:
					UTRL3 = 0x8138;
					break;
    
				default:         // set to 19200 Baud
					UTRL3 = 0x8138;
					break;
			}
    
			if(bit==8) {  
				SCR3_CL = 1; // 8 data bits 
			}
			else {	
				SCR3_CL = 0; // 7 data bits 
			}
	
			if(stop==1) {	
				SCR3_SBL = 0; // 1 stop bit 
			}
			else {
				SCR3_SBL = 1; // 2 stop bits 
			}
  
			switch(parity) {
				case 0:
					SCR3_PEN  = 0; // parity disable 
					break;
    
				case 1: 
					SCR3_P = 1; // odd parity
					SCR3_PEN = 1; // parity enable
					break;
    
				case 2:
					SCR3_P = 0; // even parity
					SCR3_PEN = 1; // parity enable
					break;
    
				default:
					SCR3_PEN = 0; // parity disable        
			}
			break; 
	}
}

/**
 * This function sets the UART interrupt enable flag.
 * @param port UART number (0..X)
 */
void initUARTir(int port) {

	switch(port) {
    	case 0:
    		SSR0_RIE=1;
			break;
    	case 1:
    		SSR1_RIE=1;
			break;
    	case 2:
    		SSR2_RIE=1;
			break;
    	case 3:
    		SSR3_RIE=1;
			break;
	}
}

/**
 * sendUART() for MB90330
 * send a byte to the UART (polling version)
 * @param port the UART you want to use (0,1,2,...)
 * @param ch the byte you want to send
 * @return None
 */ 
 
void sendUARTbyte(int port, byte ch) {         // sends a char

	switch(port) {
		case 0:
			while(SSR0_TDRE == 0);	// wait for transmit buffer empty 	
			SODR0 = ch;		//put ch into buffer   
			break;
		case 1:
			while(SSR1_TDRE == 0);	// wait for transmit buffer empty 	
			SODR1 = ch;		//put ch into buffer   
			break; 
		case 2:
			while(SSR2_TDRE == 0);	// wait for transmit buffer empty 	
			SODR2 = ch;		//put ch into buffer   
			break;
		case 3:
			while(SSR3_TDRE == 0);	// wait for transmit buffer empty 	
			SODR3 = ch;		//put ch into buffer   
			break; 
	}
}

/**
 * getUARTbyte() for MB90330
 * read a byte from UART
 * This function waits until a byte is received - if no byte is received it waits endless !!!
 * @param port the UART you want to use (0,1,2,...)
 * @return the received byte
 */

byte getUARTbyte(int port)	{ // waits for and returns incomming char 

	byte b;
  
	switch(port) {
   
		case 0:
			while(SSR0_RDRF == 0);	/* wait for data received  	*/
			if (SSR0_ORE | SSR0_FRE | SSR0_PE) {     	/* error */
				SCR0_REC  = 0; // clear error flags 
				b = SIDR0;	/* reset error flags 		*/
				return (0xFF);
			}
			else {
				return (SIDR0);	/* return char 			*/
			}
			break; 
		case 1:
			while(SSR1_RDRF == 0);	/* wait for data received  	*/
			if (SSR1_ORE | SSR1_FRE | SSR1_PE) {     	/* error */
				SCR1_REC  = 0; // clear error flags 
				b = SIDR1;	/* reset error flags 		*/
				return (0xFF);
			}
			else {
				return (SIDR1);	/* return char 			*/
			}
			break;
		case 2:
			while(SSR2_RDRF == 0);	/* wait for data received  	*/
			if (SSR2_ORE | SSR2_FRE | SSR2_PE) {     	/* error */
				SCR2_REC  = 0; // clear error flags 
				b = SIDR2;	/* reset error flags 		*/
				return (0xFF);
			}
			else {
				return (SIDR2);	/* return char 			*/
			}
			break;
		case 3:
			while(SSR3_RDRF == 0);	/* wait for data received  	*/
			if (SSR3_ORE | SSR3_FRE | SSR3_PE) {     	/* error */
				SCR3_REC  = 0; // clear error flags 
				b = SIDR3;
				return (0xFF);
			}
			else {
				return (SIDR3);	/* return char 			*/
			}
			break;

	}
	return 0; 
}