/** \file
 *
 * cs8900.c
 *
 * This file includes functions to initialize, and communicate with the CS8900A ethernet 
 * controller.
 *
 * @author  oalt
 * @version $Id: cs890090330.c,v 1.8 2005/08/08 16:12:29 fritz Exp $
 *
 */


#include "globtype.h"
#include "dos.h"
#include "cs8900.h"
#include "buffers.h"
#include "globtype.h"
#include "arp.h"
#include "uartapi.h"
#include "config.h"


extern IPRCVle* ip_write;       
extern IPRCVle* ip_read;



/*
 * ISR for CS8900 interrupt (ext. int. 0 at MCU)
 *
 */
__interrupt void intCS8900(void)  {
	
	word isq;
	word read;
	word len;
	word cnt;
	word rxstat;

	
	CS8900_IRQENABLE=0;
	
	#ifdef DEBUG_ENABLE_ALL
	printfUART(UART_DEBUG,"cs8900 interrupt\r\n");
	#endif
	
	do {
		isq=inport(IO_BASE + 8);	
	
		if((isq & 0x0104)==0x0104) { // RxOK int
			rxstat=inport(IO_BASE);
			len=inport(IO_BASE);
	  	
	  		#ifdef DEBUG_ENABLE_ALL
		  	printfUART(UART_DEBUG,"rxstat: 0x%d length: 0x%d \r\n", rxstat, len);
		  	#endif 

		  	// length=0;
		  	for(cnt=0; cnt<len; cnt+=2) {
		    	read=inport(IO_BASE);
		    	ip_write->data[cnt]=(byte)(read & 0x00FF);
	   	 		ip_write->data[cnt+1]=(byte)(read >> 8);
	    		// delay(250);
				#ifdef DEBUG_ENABLE_ALL
		    	sendUARThex(UART_DEBUG,(byte)(read & 0x00FF));
	   	 		sendUARThex(UART_DEBUG,(byte)(read >> 8));
	    		sendUARTbyte(UART_DEBUG,' ');
		    	#endif
		  	}
		  	ip_write->length=len;
		  	
			if(ip_write->next==ip_read) {
				// Overflow
				printfUART(UART_DEBUG,"cs8900 overflow, committing suicide\r\n");
				while(1);
			}
			else {
				// buffer only IP frames with indiv. addr. and broadcast ARP frames
				if((rxstat==0x0904 && ip_write->data[12]==0x08
						&& ip_write->data[13]==0x06) ||
						(rxstat==0x0504 && ip_write->data[12]==0x08
						&& ip_write->data[13]==0x00)) { // ||
//						(rxstat==0x0104 && ip_write->data[12]==0x08 // promisc mode
//						&& ip_write->data[13]==0x00))	{
					ip_write=ip_write->next;
				}
			}
		}
	} while(isq!=0);
	
	CS8900_IRQREQUEST=0;
	CS8900_IRQENABLE=1;
}