/** \file
 * 
 * tpuart90330.c
 *
 * hardware dependend part
 *
 * @version $Id: tpuart90330.c,v 1.8 2005/10/05 16:16:14 fritz Exp $
 * @author  fritz
 */

#include "globtype.h"
#include "tpuart.h"
#include "uartapi.h"
#include "mb90330.h"
#include "config.h"
#include "helper.h"

extern byte rb[20][25];		// receive queue
extern int tvals[25];
extern int row_read;
extern int col_read;
extern int row_write;
extern int col_write;

extern byte sending;
extern byte sendack;

/**
 * Init TPUART receiving timeout timer
 */
void initTimer(void) {
	
	TMRLR0=0x070C; // set reload value x 1,33us => 2,4ms
	TMCSR0=0x081A; // prescaler 1,33us at 24 MHz

}

/**
 * reset TP UART by hardware
 * @param port UART number (0..X)
 **/
void resetTPUARThw(int port) {
	byte temp;
	switch (port) {
		case 2:
			SCR2_RXE = 0; // receive disable
			SSR2_RIE = 0; // disable receive interrupt
			PDR9_P92 = 1;
			wait(50);
			SCR2_RXE = 1; // receive enable 
			PDR9_P92 = 0;
			temp = SIDR2; // read Reset.Ind
			#ifdef DEBUG_ENABLE_MOST
			printfUART(UART_DEBUG,"received 0x%x after reset\r\n",temp);
			#endif
			wait(50);
			temp = SIDR2; // read Reset.Ind
			#ifdef DEBUG_ENABLE_MOST
			printfUART(UART_DEBUG,"received 0x%x after reset\r\n",temp);
			#endif
			SCR2_REC = 0; // clear error flags 			
			break;
			
		case 3:
			PDR9_P95 = 1;
			wait(50);
			PDR9_P95 = 0;
			wait(50);
/*			temp = getUARTbyte(3); // clear error flags
			#ifdef DEBUG_ENABLE_MOST
			printfUART(UART_DEBUG,"received 0x%x after reset\r\n",temp);
			#endif*/
			break;
		
	}
}

/**
 *	returns status of save pin
 *	0 if uart connected
 *  1 if uart is disconnected
 * -1 if error
 **/
int getTPUARTSavepin(int port) {
	switch (port) {
		case 2:
			return (PDR6_P64);
			break;
		
		case 3:
			return (PDR6_P65);
			break;
	}
	return -1;
}

/**
 * ISR for UART2/3
 */
__interrupt void rcUART2_3(void) { 

	unsigned int addr;

	TMCSR0_CNTE=0; // stop timer

	if (SSR2_RDRF==1) { // receive complete UART2
		//error checking
		if (SSR2_ORE | SSR2_FRE | SSR2_PE) {     	/* error */
			rb[row_write][col_write]=0xFF;
			SCR2_REC = 0; // clear error flags 			
		}
		else {
			rb[row_write][col_write]=SIDR2;
		}
	}
	else if (SSR3_RDRF==1) { // receive complete UART3
		//error checking
		if (SSR3_ORE | SSR3_FRE | SSR3_PE) {     	/* error */
			rb[row_write][col_write]=0xFF;
			SCR3_REC = 0; // clear error flags 			
		}
		else {
			rb[row_write][col_write]=SIDR3;
		}
	}
	else {
		printfUART(UART_DEBUG,"something wrong in ISR rc_UART2_3\r\n");
		while(1);
	} 


	#ifdef DEBUG_ENABLE_ALL
		printfUART(UART_DEBUG,"TPUART: 0x");
		sendUARThex(UART_DEBUG,rb[row_write][col_write]);
		printfUART(UART_DEBUG,"\r\n");
	#endif
	
	if(col_write==4) {
		addr=rb[row_write][3]<<2 & rb[row_write][4];
		if (findknxaddress(&addr)) {
 
			if(sending==0) {
				sendUARTbyte(UART_KNX0, 0x11); // ack frame
			}
			else {
				sendack=1;
			}
		}
	}
	col_write++;    

	TMCSR0_CNTE=1; // count enable
	TMCSR0_TRG=1; // start timer
}


/**
 * ISR for receive timeout (timer 0)
 */
__interrupt void rc_timeout(void)  
{

	#ifdef DEBUG_ENABLE_ALL
	printfUART(UART_DEBUG,"receive timeout");
	#endif

	TMCSR0_INTE=0; // disable interrupt
	TMCSR0_UF=0; // clear underflow
  
	TMCSR0_CNTE=0; // count enable false
	col_write=0;
	row_write++;

	if(row_write==25) {
		row_write=0;
	}

	TMCSR0_INTE=1; // enable interrupt
}
