/** \file
 *
 * arp.c
 * This file includes functions to handle ARP requests and to manage the ARP buffer
 * @author  oalt
 * @version $Id: arp.c,v 1.6 2005/08/08 10:11:03 fritz Exp $
 *
 */
 
#include "globtype.h"
#include "arp.h"
#include "ip.h"
#include "cs8900.h"
#include "buffers.h"
#include "config.h"

extern byte send[MTU];
extern ARPentry * arpfl;
extern ARPentry * arpl;
const byte hwaddress[]={CS8900_MAC01>>8,CS8900_MAC01&0xFF, 
						CS8900_MAC23>>8,CS8900_MAC23&0xFF, 
						CS8900_MAC45>>8,CS8900_MAC45&0xFF};
const byte arpp[]={0x08, 0x06};
const byte ipp[]={0x08, 0x00};

/**
 * Call this function to send an IP frame
 *
 * @param out Byte pointer to send data
 * @param datalen frame length
 */
void sendipframe(byte* out, word datalen) {
	IPHEADER * siph;
	byte ipaddr[4];
	ARPentry * result;

	siph=(IPHEADER *) out;
	ipaddr[0]=siph->ip_daddr_3;
	ipaddr[1]=siph->ip_daddr_2;  
	ipaddr[2]=siph->ip_daddr_1;  
	ipaddr[3]=siph->ip_daddr_0;

	result=getHWA(ipaddr);
	if (result==0) {
		printfUART(UART_DEBUG,"HELP!!!no mac adress\r\n");
		// todo: send arp request
	}

	framesend((byte*)&(result->ha[0]), (byte*)hwaddress, (byte*) ipp, out, datalen);
}


/**
 * This function is called if the ethernetcontroller has accepted a frame.
 * If the frame is an ARP request (0x0806) an ARP reply will be sent.
 * If the frame is an IP frame, the ipmanager() will be called.
 */
void arpmanager(byte * rcvdata, word rcvdatalen)  {
	ARPHEADER * rarph;
  
	if (rcvdatalen <14) {
		// very basic check
		return;
  	}

	if(rcvdata[12]==0x08 & rcvdata[13]==0x00) {	
		/*
		 * network is switched
		 * so i should handle all hardware adresses i receive.
		 * if network not switched, storage of arp entries
		 * should only be done, if packet is adressed at me
		 */
	  	manageARPentry((byte*) rcvdata+14+12 , 
   	 	            (byte *) rcvdata+6);

		ipmanager((rcvdata + 14),rcvdatalen-14);
		
		printfUART(UART_DEBUG,"ipmanager exit\r\n");

	}
	else if(rcvdata[12]==0x08 & rcvdata[13]==0x06) {
		/*
		 * network is switched
		 * so i should handle all hardware adresses i receive.
		 * if network not switched, storage of arp entries
		 * should only be done, if packet is adressed at me
		 */
	  	manageARPentry((byte*) rcvdata+14+12 , 
   	 	            (byte *) rcvdata+6);

		rarph=(ARPHEADER *)(rcvdata + 14); // align ARPHEADER
    
		#ifdef DEBUG_ENABLE_ALL
			printfUART(UART_DEBUG,"opcode:0x%x%x reqip:%x.%x.%x.%x \r\n", rarph->arp_opcode_h,rarph->arp_opcode_l,
							rarph->arp_tpaddr_3,rarph->arp_tpaddr_2,rarph->arp_tpaddr_1,rarph->arp_tpaddr_0);
		#endif

		if(rarph->arp_opcode_h==0x00 && rarph->arp_opcode_l==0x01 &&
			equal(CURRENT_IP_ADDRESS,	rarph->arp_tpaddr_3,
    											rarph->arp_tpaddr_2,
    											rarph->arp_tpaddr_1,
    											rarph->arp_tpaddr_0)) {

			#ifdef DEBUG_ENABLE_ALL
				printfUART(UART_DEBUG,"yipeeee...arp request for me\r\n");
			#endif
			handle_arp_request((byte *)rarph,rcvdatalen-14);
		}
	}
}

/**
 * This function handles an ARP request.
 *
 */
void handle_arp_request(byte * rcvdata, word rcvdatalen)  {
	ARPHEADER * rarph;
	ARPHEADER * arpout;
  
	if (rcvdatalen<=0)
		return;

	rarph=(ARPHEADER *)(rcvdata); // align ARPHEADER
	arpout=(ARPHEADER *)&send[0];
  
	arpout->arp_hardware_h=0x00;
	arpout->arp_hardware_l=0x01;
	
	arpout->arp_proto_h=0x08;
	arpout->arp_proto_l=0x00;

	arpout->arp_halen=0x06;
	
	arpout->arp_protoalen=0x04;
  
	arpout->arp_opcode_h=0x00;  // reply
	arpout->arp_opcode_l=0x02;    

	arpout->arp_shaddr_5=CS8900_MAC01>>8;  // my hardware address
	arpout->arp_shaddr_4=CS8900_MAC01&0xFF;
	arpout->arp_shaddr_3=CS8900_MAC23>>8;
	arpout->arp_shaddr_2=CS8900_MAC23&0xFF;
	arpout->arp_shaddr_1=CS8900_MAC45>>8;
	arpout->arp_shaddr_0=CS8900_MAC45&0xFF;

	arpout->arp_spaddr_3=(CURRENT_IP_ADDRESS >> 24) & 0xFF;  // my IP address
	arpout->arp_spaddr_2=(CURRENT_IP_ADDRESS >> 16) & 0xFF;
	arpout->arp_spaddr_1=(CURRENT_IP_ADDRESS >> 8) & 0xFF;
	arpout->arp_spaddr_0=(CURRENT_IP_ADDRESS ) & 0xFF;

	arpout->arp_thaddr_5=rarph->arp_shaddr_5;
	arpout->arp_thaddr_4=rarph->arp_shaddr_4;
	arpout->arp_thaddr_3=rarph->arp_shaddr_3;
	arpout->arp_thaddr_2=rarph->arp_shaddr_2;
	arpout->arp_thaddr_1=rarph->arp_shaddr_1;
	arpout->arp_thaddr_0=rarph->arp_shaddr_0;

	arpout->arp_tpaddr_3=rarph->arp_spaddr_3;
	arpout->arp_tpaddr_2=rarph->arp_spaddr_2;
	arpout->arp_tpaddr_1=rarph->arp_spaddr_1;
	arpout->arp_tpaddr_0=rarph->arp_spaddr_0;

//	addARPentry((byte*)rarph+14, (byte*)rarph+8);

	framesend((byte*)rarph+8, 
				(byte *)hwaddress,
				(byte *)arpp, 
				&send[0], 28);  
}

/**
 * Call this function to get the hardwareaddress (MAC) for the given IP address.
 * @param ipadd byte array with the ip address
 * @return the valid ARPentry* or 0 if not found
 */
ARPentry* getHWA(byte* ipadd)  {
	ARPentry * run;

	run=arpl;

	while(run!=0) {
		if(ipadd[0]==run->ip[0] && ipadd[1]==run->ip[1] &&
			ipadd[2]==run->ip[2] && ipadd[3]==run->ip[3]) {
			
			return run;
		}
		run=run->next;
	}

	return 0;  // not found   
}

/**
 * Call this function to manage ARPentries in the ARP buffer
 * if hwa is not present, it will be added
 * if hwa is present, age of entry is reset
 * @param ip byte pointer of the IP address array
 * @param ha byte pointer of the MAC address associatet with the IP address   
 */
void manageARPentry(byte* ip, byte* ha)  { 
	ARPentry * tmp;
	int cnt;

	// search entry
	tmp=arpl;
	while(tmp!=0) {
		if(ip[0]==tmp->ip[0] && ip[1]==tmp->ip[1] &&
			ip[2]==tmp->ip[2] && ip[3]==tmp->ip[3]) {
			// entry found, update age and hwa
			tmp->age =0;
			for(cnt=0; cnt<6; cnt++) {
				tmp->ha[cnt]=*ha;
				ha++;
			}
			#ifdef DEBUG_ENABLE_ALL
			printfUART(UART_DEBUG,"manual updating arp entry of %i.%i.%i.%i hwa: 0x%x%x%x%x%x%x\r\n",
				tmp->ip[0],tmp->ip[1],tmp->ip[2],tmp->ip[3],
				tmp->ha[0],tmp->ha[1],tmp->ha[2],tmp->ha[3],tmp->ha[4],tmp->ha[5],tmp->ha[6]);
			#endif
			return;
		}
		tmp=tmp->next;
	}
	
	
	//entry not found -> add
	if(arpl==0) {
		arpl=arpfl;
		arpfl=arpfl->next;
		arpl->next=0;
	}
	else {
		if(arpfl!=0) { // free elements available
			tmp=arpfl;
			arpfl=arpfl->next;
			tmp->next=arpl;
			arpl=tmp;
		}
		else {  // free list empty
			// delete an entry
			// better: delete oldest entry in list
			tmp=arpl;
			while(tmp->next->next!=0) {
				tmp=tmp->next;
			}
			tmp->next->next=arpl;
			arpl=tmp->next;
			tmp->next=0;
		}
	}

	for(cnt=0; cnt<4; cnt++) {  // fill data in first element
		arpl->ip[cnt]=*ip;
		ip++;
	}

	for(cnt=0; cnt<6; cnt++) {
		arpl->ha[cnt]=*ha;
		ha++;
	}
	arpl->age=0; // reset age
	
	#ifdef DEBUG_ENABLE_ALL
	printfUART(UART_DEBUG,"adding arp entry for %i.%i.%i.%i hwa: 0x%x%x%x%x%x%x\r\n",
		arpl->ip[0],arpl->ip[1],arpl->ip[2],arpl->ip[3],
		arpl->ha[0],arpl->ha[1],arpl->ha[2],arpl->ha[3],arpl->ha[4],arpl->ha[5],arpl->ha[6]);
	#endif
	
}
