/** \file
 *
 * buffers.c
 * 
 * Ringpuffer fuer Netzwerkdaten
 *
 * @version $Id: buffers.c,v 1.1 2005/08/08 11:55:00 fritz Exp $
 */
 
#include "buffers.h"
#include "rtc.h"
#include "arp.h"
#include "config.h"

// IP receive buffers
IPRCVle iprb[RCVLISTSIZE];
IPRCVle* ip_read;
IPRCVle* ip_write;
byte send[MTU];

ARPentry arplist[ARPLISTSIZE];  // ARP entry array
ARPentry * arpfl;								// pointer to freelist
ARPentry * arpl;								// pointer to arplist

/**
 * Initialisiere den Ring.
 */
void initBuffers() {
	int i;

	// initialize ring buffer for packet receiving
	for (i=0; i<RCVLISTSIZE;i++) {
		iprb[i].next=&iprb[(i+1)%RCVLISTSIZE];	
	}  
	ip_read=&iprb[0];
	ip_write=ip_read;

	// initialize arp list
	arpfl=arplist;		// build free list
	arpl=0;				// no arp entries
	for(i=0; i<ARPLISTSIZE-1; i++) {
		arplist[i].next=&arplist[i+1];
		arplist[i].age=0;
	}
	arplist[ARPLISTSIZE-1].next=0;
	arplist[ARPLISTSIZE-1].age=0;


	if (!RTCregisterfunc(10,ARPtimeint)) {	// register callback function for timeout detection
		printfUART(UART_DEBUG, "cannot register ARP time management function\r\n");
	}
}

/**
 * delete old entries from arp list
 **/ 
void ARPtimeint(void) {
	ARPentry *tmp;
	ARPentry *prev;
	
	prev=arpl;
	tmp=arpl;
	
	while (tmp!=0) {
		// for all entries in arplist
	
		tmp->age= tmp->age+10;  // increase age

		#ifdef DEBUG_ENABLE_ALL
		printfUART(UART_DEBUG,"updating arp entry of %i.%i.%i.%i hwa: 0x%x%x%x%x%x%x age:%x\r\n",
			tmp->ip[0],tmp->ip[1],tmp->ip[2],tmp->ip[3],
			tmp->ha[0],tmp->ha[1],tmp->ha[2],tmp->ha[3],tmp->ha[4],tmp->ha[5],tmp->ha[6],
			tmp->age);
		#endif

		if (tmp->age>=ARPTIMEOUT) {	// entry is old -> delete from arpl and add to arpfl

			#ifdef DEBUG_ENABLE_ALL
			printfUART(UART_DEBUG,"deleting arp entry for %i.%i.%i.%i hwa: 0x%x%x%x%x%x%x\r\n",
				tmp->ip[0],tmp->ip[1],tmp->ip[2],tmp->ip[3],
				tmp->ha[0],tmp->ha[1],tmp->ha[2],tmp->ha[3],tmp->ha[4],tmp->ha[5],tmp->ha[6]);
			#endif
			
			if (tmp==arpl) {		// beginning of list
				arpl=tmp->next;	// set arpl to next entry
				tmp->next=arpfl;	// set next of current entry to beginning of free list
				arpfl=tmp;			// set free list to current entry
				tmp=arpl;			// set current entry to next element in arpl -> next item in loop
			}
			else {							// middle of list
				prev->next=tmp->next;	// set previous follower to follower of current item
				tmp->next=arpfl;			// set next of current entry to beginning of free list
				arpfl=tmp;					// set free list to current entry
				tmp=prev->next;			// set current entry to next element in arpl -> next item in loop	
			}
		}
		else {	// entry is not to old, next item in loop
			prev=tmp;
			tmp=tmp->next;	
		}
	}
}
