/**
 * convert.c
 *
 * helper functions for converting between emi formats
 **/
 
#include "convert.h"
#include "stddef.h"

 
/**
 * extract cemi data from receive buffer, beginning at actual start of cemi part
 **/
void extractCemi(CEMI_L_DATA_MESSAGE * cemi,byte *rcvdata) {
	cemi->mc=rcvdata[0];
	
	if (cemi->mc == L_DATA_REQ	|
		cemi->mc == L_DATA_CON |
		cemi->mc == L_DATA_IND) {

		cemi->addil=rcvdata[1];
		if (cemi->addil>0) {
			cemi->addi=&rcvdata[2];
		}
		cemi->ctrl1=rcvdata[2+cemi->addil];
		cemi->ctrl2=rcvdata[2+cemi->addil+1];
		cemi->saddr_h=rcvdata[2+cemi->addil+2];
		cemi->saddr_l=rcvdata[2+cemi->addil+3];
		cemi->daddr_h=rcvdata[2+cemi->addil+4];
		cemi->daddr_l=rcvdata[2+cemi->addil+5];
		cemi->datal=rcvdata[2+cemi->addil+6];
		cemi->data=&rcvdata[2+cemi->addil+8]; // siemens ip router sends 00 before actual data
	}
}


/**
 * initialize eib frame from cemi frame
 * attention -> direct matching of message code !!
 **/
void initEIBframeCemi(EIBFRAME* ef, CEMI_L_DATA_MESSAGE *cemi) {
	initEIBframe(	ef,
						((cemi->ctrl1)>>2) & 0x3,	// priority
						(cemi->saddr_h<<8) | (cemi->saddr_l&0xFF),
						(cemi->daddr_h<<8) | (cemi->daddr_l&0xFF),
						cemi->ctrl2>>7,				// daf
						//cemi->mc,
						0,
						cemi->data,
						cemi->datal);
}


/**
 * initialize cemi l_data frame from eib frame
 * attention -> direct matching of message code !!
 **/
void initCemiframe(CEMI_L_DATA_MESSAGE *cemi, EIBFRAME *ef) {
	cemi->mc = ef->tpci;
	cemi->addil=0;
	cemi->addi=NULL;
	cemi->ctrl1= 	(BIT7(ef->ctrl) << 7 ) |
						(0 << 6 ) |
						(BIT5(ef->ctrl) << 5 ) |
						(0 << 4) |
						(BIT3(ef->ctrl) << 3) |
						(BIT2(ef->ctrl) << 2) |
						(1 << 1) |
						(0) ;
	cemi->ctrl2=	(BIT7(ef->ll_length) << 7 ) |
						(BIT6(ef->ll_length) << 6) |
						(BIT5(ef->ll_length) << 5) |
						(BIT4(ef->ll_length) << 4);
	cemi->saddr_h= ef->saddr_h;
	cemi->saddr_l= ef->saddr_l;
	cemi->daddr_h= ef->daddr_h;
	cemi->daddr_l= ef->daddr_l;
	cemi->datal= (ef->ll_length) & 0x0F;
	cemi->data= &(ef->data[0]);
}