/** \file
 *
 * cs8900.c
 *
 * This file includes functions to initialize, and communicate with the CS8900A ethernet 
 * controller.
 *
 * @author  oalt
 * @version $Id: cs8900.c,v 1.7 2005/08/06 23:07:19 fritz Exp $
 *
 */


#include "globtype.h"
#include "dos.h"
#include "cs8900.h"
#include "buffers.h"
#include "globtype.h"
#include "arp.h"
#include "uartapi.h"
#include "config.h"
#include "helper.h"


/**
 * This function initializes the CS8900A
 * The MAC address will be set to CS8900_MAC45,CS8900_MAC23,CS8900_MAC01 and the IRQ X enabled
 */
void initCS8900()  {

	long cnt = 1000000L;
	while(cnt--);
  
	inportb(0x001);  // switch to 16 bit mode
	inportb(0x000);
	inportb(0x001);

	outport((IO_BASE + PP_POINTER), 0x158); // IEEE Address
	outport((IO_BASE + PP_DATA), SWAP(CS8900_MAC01));
 
	outport((IO_BASE + PP_POINTER), 0x15A); // IEEE Address
	outport((IO_BASE + PP_DATA), SWAP(CS8900_MAC23));
 
	outport((IO_BASE + PP_POINTER), 0x15C); // IEEE Address
	outport((IO_BASE + PP_DATA), SWAP(CS8900_MAC45));
 
	outport((IO_BASE + PP_POINTER), 0x0104 ); 	// RxCTL
	outport((IO_BASE + PP_DATA), 0x0D05 );		 	// BroadcastA, IndiviadualA, RxOKA
//	outport((IO_BASE + PP_DATA), 0x0185 );		 	// promisc
	// outport( (IO_BASE + PP_POINTER ), 0x0118 );	// TestCTL
	// outport( (IO_BASE + PP_DATA ), 0x4280 );		// Loopback Tests
	outport((IO_BASE + PP_POINTER), 0x0102);		// RxCFG
	outport( (IO_BASE + PP_DATA ), 0x0100);			// RxOKiE
   
	outport((IO_BASE + PP_POINTER), 0x0116);		// BusCTL 
	outport((IO_BASE + PP_DATA), 0x8000);			// EnableIRQ
   
	outport((IO_BASE + PP_POINTER), 0x0022);		// Interrupt pin enable 
	outport((IO_BASE + PP_DATA), CS8900_IRQ);		// Enable Pin X
   
	outport((IO_BASE + PP_POINTER), 0x0112);		// LineCTL
	outport((IO_BASE + PP_DATA), 0x00c0 );			// 10BaseT, SerRxOn, SerTxOn

	CS8900_IRQREQUEST=0;
	CS8900_LEVELA=1;  // enable ext. interrupt 0 for CS8900
	CS8900_LEVELB=0;

	while(inport(IO_BASE + 8)!=0);  // clear ISQ
	CS8900_IRQREQUEST=0;

	CS8900_IRQENABLE=1;

}

/**
 * Send a frame via Ethernet
 * @param destaddr 802.3 destination address
 * @param seraddr 802.3 source address 
 * @param proto protocol type
 * @param data data for sending
 * @param datalen data length in bytes
 */
 
void framesend(byte* destaddr, byte* srcaddr, byte* proto,
							 byte* data, word datalen)  {
  word out;
  word i;
  
  outport( (IO_BASE + TX_CMD ), 0x00c0 );  // Start Transmission after entire frame in CS8900
      
  outport( ( IO_BASE + TX_LENGTH ), datalen+14 ); // bid for bytes
      
  outport( (IO_BASE + PP_POINTER), 0x0138 );  // BusST
  // delay(25);
  if (inport( ( IO_BASE + PP_DATA ) ) !=  0x0118) // Ready4TxNOW ?
  {
    printfUART(1,"\n\n Bid for transmit failed");
    while(1);
   	// exit(FAILURE);
  }

  // copy destination address
  for(i=0; i<6; i+=2)  
  {
     out=(destaddr[i] | (destaddr[i+1]<<8));
     outport( IO_BASE , out );
  }
  // copy source address
  for(i=0; i<6; i+=2)  
  {
     out=(srcaddr[i] | (srcaddr[i+1]<<8));
     outport( IO_BASE , out );
  }
  // copy protocol type
  out=(proto[0] | (proto[1]<<8));
  outport( IO_BASE, out);
  
  // Copy the TX frame to the on-chip buffer 
	// Write XX/2 words to IOBase+0x0 (TX Data Port) 

  for (i = 0; i < datalen; i+=2)
  {
    out=(data[i] | (data[i+1]<<8));
   	outport( IO_BASE , out );
  }
 
  
  
}