/*#######################################################################################
FAT for AVR (MMC/SD) 

Copyright (C) 2004 Ulrich Radig

Bei Fragen und Verbesserungen wendet euch per EMail an

mail@ulrichradig.de

oder im Forum meiner Web Page : www.ulrichradig.de


Dieses Programm ist freie Software. Sie k nnen es unter den Bedingungen der 
GNU General Public License, wie von der Free Software Foundation ver ffentlicht, 
weitergeben und/oder modifizieren, entweder gem   Version 2 der Lizenz oder 
(nach Ihrer Option) jeder sp teren Version. 

Die Ver ffentlichung dieses Programms erfolgt in der Hoffnung, 
da  es Ihnen von Nutzen sein wird, aber OHNE IRGENDEINE GARANTIE, 
sogar ohne die implizite Garantie der MARKTREIFE oder der VERWENDBARKEIT 
F R EINEN BESTIMMTEN ZWECK. Details finden Sie in der GNU General Public License. 

Sie sollten eine Kopie der GNU General Public License zusammen mit diesem 
Programm erhalten haben. 
Falls nicht, schreiben Sie an die Free Software Foundation, 
Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA. 
#######################################################################################*/

#include "fat.h"
#include "mmc.h"

// zum Simulieren des EEPROMs (wg. Komp. zum AVR)
#define EE_CLU_SIZE   (&DUMMY_EEPROM[0])
#define EE_CLU_OFFSET (&DUMMY_EEPROM[2])
#define EE_VBRA       (&DUMMY_EEPROM[4])
#define EE_FAT_OFFSET (&DUMMY_EEPROM[6])

unsigned char DUMMY_EEPROM[10];  

#define eeprom_read_byte(a)       (unsigned char)*(a)
#define eeprom_write_byte(a,b)    *a = b

#define eeprom_read_word(a)       (unsigned int)*(a)
#define eeprom_write_word(a,b)    *a = b


//############################################################################
//Auslesen der Adresse des Volume Boot Record von MBR
unsigned int FAT_Addr (unsigned char *Buffer)
//############################################################################
{
	unsigned int *Volume_Boot_Record_Addr;

	//auslesen des Master Boot Record von der MMC/SD Karte (addr = 0)
	mmc_read_sector (MasterBootRecord,Buffer); //Read Master Boot Record

	Volume_Boot_Record_Addr = (unsigned int *)&Buffer[VBR_Addr];
	//Berechnet Volume Boot Record 

	#ifdef DEBUG_ENABLE_ALL
		printfUART(UART_DEBUG,"VBR_Addr: 0x%x%x \r\n",*Volume_Boot_Record_Addr>>8,*Volume_Boot_Record_Addr&0xFF);
	#endif

    return (*Volume_Boot_Record_Addr);
}
//############################################################################
//Auslesen der Adresse des First Root Directory von Volume Boot Record
unsigned int Root_Dir_Addr (unsigned char *Buffer) 
//############################################################################
{
	struct BootSec *bootp; //Zeiger auf Bootsektor Struktur
	unsigned int FirstRootDirSecNum;

	unsigned int Volume_Boot_Record_Addr = eeprom_read_word ((unsigned int*)EE_VBRA); //0x3c
	//auslesen des Volume Boot Record von der MMC/SD Karte 
	mmc_read_sector (Volume_Boot_Record_Addr,Buffer);
	bootp=(struct BootSec *)Buffer;

	//berechnet den ersten Sector des Root Directory
	FirstRootDirSecNum = ( bootp->BPB_RsvdSecCnt +  // bit17+18 0x0100
	                       (bootp->BPB_NumFATs * bootp->BPB_FATSz16) //bit 19+25/26  0x02 * 0x0002
			     );

	FirstRootDirSecNum+= Volume_Boot_Record_Addr;
	
	#ifdef DEBUG_ENABLE_ALL
		printfUART(UART_DEBUG,"FirstRootDirSecNum: 0x%x%x\r\n",FirstRootDirSecNum>>8,FirstRootDirSecNum&0xFF);
	#endif
	
	return(FirstRootDirSecNum);
}

//############################################################################
//	Ausgabe des angegebenen Directory Eintrag in Entry_Count
//	ist kein Eintrag vorhanden, ist der Eintrag im 
//	Rckgabe Cluster 0xFFFF. Es wird immer nur ein Eintrag ausgegeben
//	um Speicherplatz zu sparen um es auch fr kleine Atmels zu benutzen
unsigned int Read_Dir_Ent (unsigned int dir_cluster, //Angabe Dir Cluster
							unsigned char Entry_Count,	//Angabe welcher Direintrag
							unsigned long *Size, 		//Rckgabe der File Gr  e
							unsigned char *Dir_Attrib,//Rckgabe des Dir Attributs
							unsigned char *Buffer) 	//Working Buffer
//############################################################################
{
	unsigned char *pointer;
	unsigned int TMP_Entry_Count = 0;
	unsigned long Block = 0;
	struct DirEntry *dir; //Zeiger auf einen Verzeichniseshort intrag

	unsigned int ClusterOffset;
	unsigned char ClusterSize;
	unsigned int blk;
	unsigned int a;
	unsigned char b;

	pointer = Buffer;

	if (dir_cluster == 0)
		{
		Block = Root_Dir_Addr(Buffer);
		#ifdef DEBUG_ENABLE_ALL
			printfUART(UART_DEBUG,"Block/RootDirAddr: 0x%x%x%x%x\r\n",Block>>24,Block>>16,Block>>8,Block&0xFF);
			if (Block==0) {
				printfUART(UART_DEBUG,"hurra\r\n");
			}
		#endif
		}
	else
		{
		ClusterOffset = eeprom_read_word ((int*)EE_CLU_OFFSET);
		ClusterSize = eeprom_read_byte ((char*)EE_CLU_SIZE);
		//Berechnung des Blocks aus BlockCount und Cluster aus FATTabelle
		//Berechnung welcher Cluster zu laden ist
		//Auslesen der FAT - Tabelle
		Load_FAT (dir_cluster,&Block,Buffer);			 
		Block = ((Block-2) * ClusterSize) + ClusterOffset;
		#ifdef DEBUG_ENABLE_ALL
			printfUART(UART_DEBUG,"calculated Block: 0x%x%x%x%x: 0x",Block>>24,Block>>16,Block>>8,Block&0xFF);
		#endif
		}

	//auslesen des gesamten Root Directory
	for (blk = Block;;blk++)
	{
		mmc_read_sector (blk,&Buffer[0]);	//Lesen eines Blocks des Root Directory
		
		#ifdef DEBUG_ENABLE_ALL
			printfUART(UART_DEBUG,"Read_Dir_Entr at blk:0x%i: 0x",blk);
			for (a=0;a<512;a++) {
				sendUARTbyte(UART_DEBUG,Buffer[a]);
			}
			printfUART(UART_DEBUG,"\r\n");
		#endif
		
		for (a=0;a<BlockSize; a = a + 32)
		{
		 dir=(struct DirEntry *)&Buffer[a]; //Zeiger auf aktuellen Verzeichniseshort intrag holen
		 
			if (dir->DIR_Name[0] == 0) //Kein weiterer Eshort intrag wenn erstes Zeichen des Namens 0 ist
			{
			return (0xFFFF);
			}
			
			//Prfen ob es ein 8.3 Eshort intrag ist
			//Das ist der Fall wenn es sich nicht um einen Eshort intrag fr lange Dateinamen
			//oder um einen als gel scht markierten Eshort intrag handelt.
   			if ((dir->DIR_Attr != ATTR_LONG_NAME) &&
				(dir->DIR_Name[0] != DIR_ENTRY_IS_FREE)) 
			{
				//Ist es der gewnschte Verzeichniseshort intrag
				if (TMP_Entry_Count == Entry_Count) 
				{
					//Speichern des Verzeichnis Eshort intrages in den Rckgabe Buffer
					for(b=0;b<11;b++)
					{
					if (dir->DIR_Name[b] != SPACE)
						{
						if (b == 8)
							{
							*pointer++= '.';
							}
						*pointer++=dir->DIR_Name[b];
						}
					}						
					*pointer++='\0';
					*Dir_Attrib = dir->DIR_Attr;

					//Speichern der Filegr  e
					*Size=dir->DIR_FileSize;
					
					//Speichern des Clusters des Verzeichniseshort intrages
					dir_cluster = dir->DIR_FstClusLO;

					//Eshort intrag gefunden Rcksprung mit Cluster File Start
					return(dir_cluster);
				}
			TMP_Entry_Count++;
			}
		}
	}
	return (0xFFFF); //Kein Eintrag mehr gefunden Rcksprung mit 0xFFFF
}

//############################################################################
//	Auslesen der Cluster fr ein File aus der FAT
//	in den Buffer(512Byte). Bei einer 128MB MMC/SD 
//	Karte ist die Cluster gr  e normalerweise 16KB gro 
//	das bedeutet das File kann max. 4MByte gro  sein.
//	Bei gr  eren Files mu  der Buffer gr  er definiert
//	werden! (Ready)
//	Cluster = Start Clusterangabe aus dem Directory	
void Load_FAT (	unsigned int Cluster, 		//Angabe Startcluster
				unsigned long *Block,
				unsigned char *TMP_Buffer) 	//Workingbuffer
//############################################################################
{
	//Zum  berprfen ob der FAT Block schon geladen wurde
	unsigned short int FAT_Block_Store = 0;	

	//Byte Adresse innerhalb des Fat Blocks
	unsigned short int FAT_Byte_Addresse;	

	//FAT Block Adresse
	unsigned short int FAT_Block_Addresse;
	
	//Berechnung fr den ersten FAT Block (FAT Start Addresse)
	unsigned int Volume_Block_Start = eeprom_read_word ((unsigned int*)EE_VBRA);
	unsigned int FAT_Offset = eeprom_read_word((unsigned int*)EE_FAT_OFFSET);
	
	unsigned int a;
	
	for (a = 0;;a++)
	{	
		if (a == *Block)
			{
			*Block = (0x0000FFFF & Cluster);
			return;
			}
		
		if (Cluster == 0xFFFF)
			{
			break; //Ist das Ende des Files erreicht Schleife beenden
			}
		//Berechnung des Bytes innerhalb des FAT Block s
		FAT_Byte_Addresse = (Cluster*2) % BlockSize;
			
		//Berechnung des Blocks der gelesen werden mu 
		FAT_Block_Addresse = ((Cluster*2) / BlockSize) + 
								Volume_Block_Start + FAT_Offset;	
		//Lesen des FAT Blocks
		// berprfung ob dieser Block schon gelesen wurde
		if (FAT_Block_Addresse != FAT_Block_Store)
			{
			FAT_Block_Store = FAT_Block_Addresse;
			//Lesen des FAT Blocks
			mmc_read_sector (FAT_Block_Addresse,TMP_Buffer);	
			}

		//Lesen der nchsten Clusternummer
		Cluster = (TMP_Buffer[FAT_Byte_Addresse + 1] << 8) + 
					TMP_Buffer[FAT_Byte_Addresse];		
	}
	return;
}

//############################################################################
//Auslesen Cluster Size der MMC/SD Karte und Speichern der gr  e ins EEprom
//Auslesen Cluster Offset der MMC/SD Karte und Speichern der gr  e ins EEprom
void Cluster_Data_Store (void)
//############################################################################
{
	#ifdef DEBUG_ENABLE_ALL
		int a=0;
	#endif

	struct BootSec *bootp; //Zeiger auf Bootsektor Struktur

	unsigned char Buffer[BlockSize];

	unsigned int Volume_Boot_Record_Addr = FAT_Addr (Buffer);
	
	unsigned int ClusterOffset;
	
	eeprom_write_word((unsigned int*)EE_VBRA,Volume_Boot_Record_Addr); //3c=7800 EE_VBRA=(&DUMMY_EEPROM[4])
   
	mmc_read_sector (Volume_Boot_Record_Addr,Buffer);

	bootp=(struct BootSec *)Buffer;

	eeprom_write_byte((unsigned char*)EE_CLU_SIZE,bootp->BPB_SecPerClus); //15.tes byte EE_CLU_SIZE=(&DUMMY_EEPROM[0])

	eeprom_write_word((unsigned int*)EE_FAT_OFFSET,bootp->BPB_RsvdSecCnt);//16.+17. byte EE_FAT_OFFSET=(&DUMMY_EEPROM[6])

	ClusterOffset = ((bootp->BPB_BytesPerSec * 32)/BlockSize);	
	ClusterOffset += Root_Dir_Addr(Buffer);
	eeprom_write_word((unsigned int*)EE_CLU_OFFSET,ClusterOffset); // EE_CLU_OFFSET=(&DUMMY_EEPROM[2])
	
	#ifdef DEBUG_ENABLE_ALL
		printfUART(UART_DEBUG,"Cluster info: 0x");
		for (a=0;a<10;a++) {
			printfUART(UART_DEBUG,"%x ",DUMMY_EEPROM[a]);
		}
		printfUART(UART_DEBUG,"\r\n");
	#endif
}

//############################################################################
//Lesen eines 512Bytes Bolckes von einem File
void Read_File (	unsigned int Cluster, 		//Angabe des Startclusters vom File
					unsigned char *Buffer,	//Workingbuffer
					unsigned long BlockCount)	//Angabe welcher Bock vom File geladen 
												//werden soll a 512 Bytes
//############################################################################
{

	unsigned long Block;
	
	// Berechnet den Block im Filesystem
	Block= getFatBlockAdr(Cluster,Buffer,BlockCount);

	mmc_read_sector (Block,Buffer);	
	return;
}

//############################################################################
//Ermitteln der nchten FAT- Blockes: bzw. Schreib/Lese-Adresse
unsigned long getFatBlockAdr (	unsigned int Cluster, 	//Angabe des Startclusters vom File
					unsigned char *Buffer,		//Workingbuffer
					unsigned long BlockCount)		//Angabe welcher Bock vom File geladen 
													//werden soll a 512 Bytes
//############################################################################
{
	unsigned int ClusterOffset = eeprom_read_word ((int *)EE_CLU_OFFSET);
	unsigned char ClusterSize  = eeprom_read_byte ((char*)EE_CLU_SIZE);

	//Berechnung des Blocks aus BlockCount und Cluster aus FATTabelle
	//Berechnung welcher Cluster zu laden ist
	
	unsigned long Block = (BlockCount/ClusterSize);
	
	//Auslesen der FAT - Tabelle
	Load_FAT (Cluster,&Block,Buffer);			 
	Block = ((Block-2) * ClusterSize) + ClusterOffset;
	//Berechnung des Blocks innerhalb des Cluster
	Block += (BlockCount % ClusterSize);

	return Block;
}

//####################################################################################
//Sucht ein File im Directory
unsigned char Search_File (unsigned char *File_Name,	//Name des zu suchenden Files
							unsigned int *Cluster, 	//Angabe Dir Cluster welches
														//durchsucht werden soll
														//und Rckgabe des clusters
														//vom File welches gefunden
														//wurde
							unsigned long *Size, 		//Rckgabe der File Gr  e
							unsigned char *Dir_Attrib,//Rckgabe des Dir Attributs
							unsigned char *Buffer) 	//Working Buffer
//####################################################################################
{
	unsigned int Dir_Cluster_Store = *Cluster;
	unsigned char a;
	for (a = 0;a < 100;a++)
	{
		*Cluster = Read_Dir_Ent(Dir_Cluster_Store,a,Size,Dir_Attrib,Buffer);

		if (*Cluster == 0xffff)
			{
			return(0); //File not Found
			}
		if(strcmp(File_Name,Buffer) == 0)
			{
			return(1); //File Found
			}
	}
	return(2); //Error
}




