/** \file
 *
 * icmp.c
 * Implementierung der ICMP Echo Funktion
 * 
 * @verion $Id: icmp.c,v 1.3 2005/08/08 10:11:03 fritz Exp $
 * @author oalt
 */
 
#include "icmp.h"
#include "ip.h"
#include "tcp.h"
#include "buffers.h"

extern TCPCON con;
extern byte send[MTU];

/**
 * call type handler functions
 * @param datalen ICMP header + data, length in bytes
 */
void icmpmanager(byte *rcvdata, word datalen)  
{
  
  ICMPHEADER * ricmph;
  
  ricmph=(ICMPHEADER *)&(rcvdata[((rcvdata[0]&0x0F)<<2)]); // align ICMPHEADER

  switch(ricmph->icmp_type)
  {
    case 8:
    	icmp_echo_reply((byte *)rcvdata, datalen);
    break;
  }
}

/** 
 * Echo reply Dienst von ICMP (ping)
 */
void icmp_echo_reply(byte *rcvdata, word datalen)  
{
  
  word cnt;
  word chksum;
  
  IPHEADER   * riph;
  ICMPHEADER * ricmph;
  ICMPECHO   * ricmpecho;
  
  IPHDATA      ip_out;
  ICMPHEADER * icmp_out;
  ICMPECHO   * icmp_echo_out;
  
  riph=(IPHEADER *)rcvdata; // align IPHEADER
  ricmph=(ICMPHEADER *)&(rcvdata[((rcvdata[0]&0x0F)<<2)]); // align TCPHEADER
  ricmpecho=(ICMPECHO *)&(rcvdata[((rcvdata[0]&0x0F)<<2)+4]);
  
  icmp_out=(ICMPHEADER *)&send[100];
  icmp_echo_out=(ICMPECHO *)&send[104];
  
  icmp_out->icmp_type=0; // Echo reply
  
  icmp_out->icmp_code=0;
  
  icmp_out->icmp_chksum_h=0;
  icmp_out->icmp_chksum_l=0;
  
  icmp_echo_out->icmp_echo_id_h = ricmpecho->icmp_echo_id_h;
  icmp_echo_out->icmp_echo_id_l = ricmpecho->icmp_echo_id_l;

  icmp_echo_out->icmp_echo_seq_h = ricmpecho->icmp_echo_seq_h;
  icmp_echo_out->icmp_echo_seq_l = ricmpecho->icmp_echo_seq_l;

  for(cnt=0; cnt<=datalen-8; cnt++)  // copy data
  {
    send[108+cnt]=rcvdata[((rcvdata[0]&0x0F)<<2)+8+cnt];
  }  
  
  // claculate checksum
  chksum=icmp_chksum(icmp_out, datalen);
  icmp_out->icmp_chksum_l=(byte)((chksum>>8) & 0x00FF);
  icmp_out->icmp_chksum_h=(byte)(chksum & 0x00FF);
  
  ip_out.ip_protocol=1; // ICMP
  ip_out.ip_daddr_3=riph->ip_saddr_3;
  ip_out.ip_daddr_2=riph->ip_saddr_2;
  ip_out.ip_daddr_1=riph->ip_saddr_1;
  ip_out.ip_daddr_0=riph->ip_saddr_0;

  ipsend(datalen, ip_out);

}

/**
 * Berechne ICMP Header Pruefsumme.
 */
word icmp_chksum(ICMPHEADER * icmph, word len)  
{

  word cnt=len;
  word * w = (word *)icmph;
  dword sum=0;
  word answer;
  
  while(cnt > 1)
  {
    sum += *w++;
    cnt -= 2;
  }
  if(cnt==1)
  {
    *(byte *)(&answer) = * (byte *)w;
    sum+=answer;
  } 
  sum = (sum >> 16) + (sum & 0xFFFF);
  sum += (sum >> 16);
  answer = ~sum;
  return (answer);
  
}