/** \file
 * ip.c
 *
 *
 * @version $Id: ip.c,v 1.5 2005/08/08 16:12:29 fritz Exp $
 * @author oalt
 */

#include "ip.h"
#include "config.h"
#include "tcp.h"
#include "udp.h"
#include "slip.h"
#include "buffers.h"
#include "eibnetip.h"
#include "helper.h"

extern byte send[MTU];
extern TCPCON con;


/**
 * Diese Funktion behandelt eingehende IP Pakete und reagiert entsprechend dem 
 * gewnschten Protokoll der darueberliegenden Schicht.
 * Implementiert sind ICMP, TCP und UDP.
 */
void ipmanager(byte *rcvdata, word datal) {

	IPHEADER * riph;
	word datalen;

	riph=(IPHEADER *)rcvdata; // align IPHEADER

  
	#ifdef DEBUG_ENABLE_MOST
		printfUART(UART_DEBUG,"ipVandLen: 0x%x\r\n",riph->ip_VandLen);
	#endif

	datalen=(word)riph->ip_length_h;  // total length
	datalen = datalen<<8;				
	datalen |= (word) (riph->ip_length_l);	
	datalen -= ((riph->ip_VandLen & 0x0F) *4); // datalen = total length - ip header length

	if (datal<datalen) {
		printfUART(UART_DEBUG,"ipmanager: something wrong with datal and datalen\r\n");
		return;
	}

	#ifdef DEBUG_ENABLE_MOST
		printfUART(UART_DEBUG,"to: %x.%x.%x.%x\r\n"	,riph->ip_daddr_3
													,riph->ip_daddr_2
													,riph->ip_daddr_1
													,riph->ip_daddr_0);
	#endif  


	// check IP destination address
	if (	equal(CURRENT_IP_ADDRESS,
					riph->ip_daddr_3, 
					riph->ip_daddr_2,
					riph->ip_daddr_1, 
					riph->ip_daddr_0) ||
			equal(SYSTEM_SETUP_MULTICAST_ADDRESS,
					riph->ip_daddr_3,
					riph->ip_daddr_2,
					riph->ip_daddr_1,
					riph->ip_daddr_0)) {

		
		// cheksum control toDO !!

    	switch(riph->ip_protocol) { 
			case 1: // ICMP
				#ifdef DEBUG_ENABLE_LITTLE
					printfUART(UART_DEBUG,"ICMP Packet for me\r\n");
				#endif
				icmpmanager(rcvdata,datalen);
				break;
			case 6: // TCP
				#ifdef DEBUG_ENABLE_LITTLE
					printfUART(UART_DEBUG,"TCP Packet for me\r\n");
				#endif
				con.ipaddr_3=riph->ip_saddr_3; // change for more than 1 socket !!!
				con.ipaddr_2=riph->ip_saddr_2;
				con.ipaddr_1=riph->ip_saddr_1;
				con.ipaddr_0=riph->ip_saddr_0;
				tcpmanager(rcvdata,datalen); // call with tcpdatalen header + data !!
				printfUART(UART_DEBUG,"tcpmanager exit\r\n");
				break;
			case 17: // UDP
				#ifdef DEBUG_ENABLE_LITTLE
					printfUART(UART_DEBUG,"UDP Packet for me\r\n");
				#endif
				udpmanager(rcvdata,datalen);
				break;
		}
	}
	// Packet not for me, return.  
}

/**
 * Prepare IP Header for sending frame
 * Funktion wird von der darueberliegenden Protokollschicht aufgerufen und 
 * baut den IP Header auf.
 *
 */
void ipsend(word datalen, IPHDATA iphd)  
{
/* #ifdef DEBUG_ENABLE_ALL
  int i;
#endif */

  //word chk;
  
  IPHEADER * out;
  
  out=(IPHEADER *)&send[80];  // send without options
  
  out->ip_VandLen=0x45;
  
  out->ip_tos=0;
  
  datalen+=20;
  
  out->ip_length_h=(byte)(datalen>>8);
  out->ip_length_l=(byte)(datalen & 0x00FF);
  
  out->ip_id_h=0x01;
  out->ip_id_l=0x00;
  
  out->ip_fragment_h=0x40; // don t fragment
  out->ip_fragment_l=0;
  
  out->ip_livetime=32;
  
  out->ip_protocol=iphd.ip_protocol; // e.g. TCP
  
  out->ip_chksum_h=0;
  out->ip_chksum_l=0;
  
  out->ip_saddr_3=CURRENT_IP_ADDRESS>>24 & 0xFF;
  out->ip_saddr_2=CURRENT_IP_ADDRESS>>16 & 0xFF;
  out->ip_saddr_1=CURRENT_IP_ADDRESS>>8  & 0xFF;
  out->ip_saddr_0=CURRENT_IP_ADDRESS     & 0xFF;
  
  out->ip_daddr_3=iphd.ip_daddr_3;
  out->ip_daddr_2=iphd.ip_daddr_2;
  out->ip_daddr_1=iphd.ip_daddr_1;
  out->ip_daddr_0=iphd.ip_daddr_0;

  ipchksum(out);
  
  // chk=i_chksum((word *)out, datalen);
  // out->ip_chksum_h=(byte)chk & 0x00FF;
  // out->ip_chksum_l=(byte)((chk >> 8) & 0x00FF);
  
  // debug code *******************************
/*#ifdef DEBUG_ENABLE_ALL
  printfUART(UART_DEBUG,"Answer Frame: %i\r\n",datalen);
  for(i=0; i<datalen; i++)
  {
    sendUARThex(UART_DEBUG,(byte)*((byte *)out + i));
    sendUARTbyte(UART_DEBUG,' ');
  }
#endif*/
  // end debug code ***************************
  
#ifdef SLIP_SEND
  slip_send((byte *)out,datalen); // send entire frame
#endif
#ifdef LAN_SEND
	sendipframe((byte*) out, datalen);
	printfUART(UART_DEBUG, "sending done...\r\n");
#endif
   

}

/**
 *
 *
 */
word i_chksum(word * data, word len)  {

  word cnt=len;
  word * w = data;
  dword sum=0;
  word answer;
  
  while(cnt > 1)
  {
    sum += *w++;
    cnt -= 2;
  }
  if(cnt==1)
  {
    *(byte *)(&answer) = * (byte *)w;
    sum+=answer;
  } 
  sum = (sum >> 16) + (sum & 0xFFFF);
  sum += (sum >> 16);
  answer = ~sum;
  return (answer);
  
}
/**
 * Calculate IP Header Checksum
 * 
 * Berechnet die IP Pruefsumme. 
 * Die implementierung ist vermutlich fehlerhaft und nicht allgemein gueltig.
 *
 */ 
void ipchksum(IPHEADER * out)  
{
  int cnt=0;
  word sum=0;
  byte * bptr;
  
  bptr=(byte *)out;
  
  // Berechne Pruefsumme
  for(cnt=0; cnt<(((out->ip_VandLen) & 0x0F)<<2); cnt+=2)
  {
    sum+=(word)(bptr[cnt]<<8 | (bptr[cnt+1] & 0x00FF));
  }
  
  sum=~sum;
  sum-=2;
  
  // Schreibe Pruefsumme in den IP Header
  out->ip_chksum_h=(byte)(sum>>8);
  out->ip_chksum_l=(byte)(sum & 0x00FF);

}
