/*#######################################################################################
Connect AVR to MMC/SD 

Copyright (C) 2004 Ulrich Radig
adapted to Fujitsu MC by Fritz Praus


Bei Fragen und Verbesserungen wendet euch per EMail an

mail@ulrichradig.de

oder im Forum meiner Web Page : www.ulrichradig.de


Dieses Programm ist freie Software. Sie k nnen es unter den Bedingungen der 
GNU General Public License, wie von der Free Software Foundation ver ffentlicht, 
weitergeben und/oder modifizieren, entweder gem  Version 2 der Lizenz oder 
(nach Ihrer Option) jeder spteren Version. 

Die Ver ffentlichung dieses Programms erfolgt in der Hoffnung, 
da  es Ihnen von Nutzen sein wird, aber OHNE IRGENDEINE GARANTIE, 
sogar ohne die implizite Garantie der MARKTREIFE oder der VERWENDBARKEIT 
F R EINEN BESTIMMTEN ZWECK. Details finden Sie in der GNU General Public License. 

Sie sollten eine Kopie der GNU General Public License zusammen mit diesem 
Programm erhalten haben. 
Falls nicht, schreiben Sie an die Free Software Foundation, 
Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA. 
#######################################################################################*/

#include "mmc.h"


//############################################################################
//Routine zur Initialisierung der MMC/SD-Karte (SPI-MODE)
unsigned char mmc_init ()
//############################################################################
{
	byte Timeout = 0;
	dword a = 0;
	byte b = 0;

	byte CMD[] = {0x40,0x00,0x00,0x00,0x00,0x95};
	
	MMC_Disable(); //Setzt den Pin MMC_Chip_Select auf High Pegel

	for(a=0; a<20000; a++) {
		nop();
	};		//Wartet eine kurze Zeit


	//Initialisiere MMC/SD-Karte in den SPI-Mode
	for (b = 0; b<10; b++) { //Sendet min 74+ Clocks an die MMC/SD-Karte
		Write_Byte_MMC(0xff);
	}
	
	//Sendet Commando CMD0 an MMC/SD-Karte
	//reset cd card
	while(Write_Command_MMC (CMD) !=1)
	{
		if (Timeout++ > 200)
			{
				return(1); //Abbruch bei Commando1 (Return Code1)
			}
	}
	
	//Sendet Commando Init(CMD1) an MMC/SD-Karte
	Timeout = 0;
	CMD[0] = 0x41;//Commando 1
	CMD[5] = 0xFF;
	while( Write_Command_MMC (CMD) !=0)
	{
		if (Timeout++ > 200)
			{
			return(2); //Abbruch bei Commando2 (Return Code2)
			}
	}
	
	//set MMC_Chip_Select to high (MMC/SD-Karte Inaktiv)
	MMC_Disable();
	return(0);
}

//############################################################################
//Sendet ein Commando an die MMC/SD-Karte
unsigned char Write_Command_MMC (unsigned char *CMD)
//############################################################################
{
	byte tmp = 0xff;
	unsigned int Timeout = 0;
	byte a = 0;

	//set MMC_Chip_Select to high (MMC/SD-Karte Inaktiv) 
	MMC_Disable();

	//sendet 8 Clock Impulse
	Write_Byte_MMC(0xFF);

	//set MMC_Chip_Select to low (MMC/SD-Karte Aktiv)
	MMC_Enable();

	//sendet 6 Byte Commando
	for (a = 0; a<0x06; a++) //sendet 6 Byte Commando zur MMC/SD-Karte
		{
		Write_Byte_MMC(*CMD++);
		}

	//Wartet auf ein gltige Antwort von der MMC/SD-Karte
	while (tmp == 0xff)	
		{
		tmp = Read_Byte_MMC();
		if (Timeout++ > 500)
			{
			break; //Abbruch da die MMC/SD-Karte nicht Antwortet
			}
		}

	return(tmp);
}

//############################################################################
//Routine zum Empfangen eines Bytes von der MMC-Karte 
byte Read_Byte_MMC (void) {
//############################################################################
	unsigned char data = 0;
	byte a = 0;

	//Routine fr Software SPI
	for (a=8; a>0; a--) //das Byte wird Bitweise nacheinander Empangen MSB First
		{
		SPI_Clock = 0; //erzeugt ein Clock Impuls (Low) 
		
		if (SPI_DI == 1) //Lesen des Pegels von MMC_DI
				{
				data |= (1<<(a-1));
				}
			else
				{
				data &=~(1<<(a-1));
				}

		SPI_Clock = 1; //setzt Clock Impuls wieder auf (High)		
		}
		
	return (data);
}

//############################################################################
//Routine zum Senden eines Bytes zur MMC-Karte
void Write_Byte_MMC (unsigned char Byte)
//############################################################################
{
	byte a = 0;
	//Routine fr Software SPI
	for (a=8; a>0; a--) //das Byte wird Bitweise nacheinander Gesendet MSB First
	{
		if (BIT(Byte,(a-1))>0)	//Ist Bit a in Byte gesetzt
				{
				SPI_DO=1; //Set Output High
				}
			else
				{
				SPI_DO=0; //Set Output Low
				}	
		SPI_Clock=0;	//erzeugt ein Clock Impuls (LOW)
		
		SPI_Clock=1; //setzt Clock Impuls wieder auf (High)

	}
	SPI_DO=1;	//setzt Output wieder auf High	

}

//############################################################################
//Routine zum schreiben eines Blocks(512Byte) auf die MMC/SD-Karte
unsigned char mmc_write_sector (unsigned long addr,unsigned char *Buffer)
//############################################################################
{
	unsigned char tmp;
	//Commando 24 zum schreiben eines Blocks auf die MMC/SD - Karte
	unsigned char CMD[] = {0x58,0x00,0x00,0x00,0x00,0xFF}; 
	byte a = 0;
	unsigned int b = 0;
	
	/*Die Adressierung der MMC/SD-Karte wird in Bytes angegeben,
	  addr wird von Blocks zu Bytes umgerechnet danach werden 
	  diese in das Commando eingefgt*/

	addr = addr << 9; //addr = addr * 512

	  
   CMD[1] = ((addr & 0xFF000000) >>24 );
   CMD[2] = ((addr & 0x00FF0000) >>16 );
   CMD[3] = ((addr & 0x0000FF00) >>8 ); 
   
   
	//Sendet Commando CMD24 an MMC/SD-Karte (Write 1 Block/512 Bytes)
	tmp = Write_Command_MMC (CMD);
	if (tmp != 0)
			{
			return(tmp);
			}
			
	//Wartet einen Moment und sendet einen Clock an die MMC/SD-Karte
	for (a=0;a<100;a++)
	{
	Read_Byte_MMC();
	}
	
	//Sendet Start Byte an MMC/SD-Karte
	Write_Byte_MMC(0xFE);	
	
	//Schreiben des Bolcks (512Bytes) auf MMC/SD-Karte
	for (b=0;b<512;b++)
		{
		Write_Byte_MMC(*Buffer++);
		}
	
	//CRC-Byte schreiben
	Write_Byte_MMC(0xFF); //Schreibt Dummy CRC
	Write_Byte_MMC(0xFF); //CRC Code wird nicht benutzt
	
	//Wartet auf MMC/SD-Karte Bussy
	while (Read_Byte_MMC() != 0xff){};
	
	//set MMC_Chip_Select to high (MMC/SD-Karte Inaktiv)
	MMC_Disable();
	return(0);
}

//############################################################################
//Routine zum lesen des CID Registers von der MMC/SD-Karte (16Bytes)
void MMC_Read_Block(unsigned char *CMD,unsigned char *Buffer,unsigned int Bytes)
//############################################################################
{	
	unsigned int a=0;
	
	//Sendet Commando CMD an MMC/SD-Karte
	if (Write_Command_MMC (CMD) != 0)
			{
			return;
			}
			
	//Wartet auf Start Byte von der MMC/SD-Karte (FEh/Start Byte)
	while (Read_Byte_MMC() != 0xfe){};
	
	//Lesen des Bolcks (normal 512Bytes) von MMC/SD-Karte
	for (a=0;a<Bytes;a++)
		{
		*Buffer++ = Read_Byte_MMC();
		}
	
	//CRC-Byte auslesen
	Read_Byte_MMC();//CRC - Byte wird nicht ausgewertet
	Read_Byte_MMC();//CRC - Byte wird nicht ausgewertet
	
	//set MMC_Chip_Select to high (MMC/SD-Karte Inaktiv)
	MMC_Disable();
	
	return;
}

//############################################################################
//Routine zum lesen eines Blocks(512Byte) von der MMC/SD-Karte
unsigned char mmc_read_sector (unsigned long addr,unsigned char *Buffer)
//############################################################################
{	
	//Commando 16 zum lesen eines Blocks von der MMC/SD - Karte
	unsigned char CMD[] = {0x51,0x00,0x00,0x00,0x00,0xFF}; 
	
	/*Die Adressierung der MMC/SD-Karte wird in Bytes angegeben,
	  addr wird von Blocks zu Bytes umgerechnet danach werden 
	  diese in das Commando eingefgt*/
	  
	addr = addr << 9; //addr = addr * 512

	CMD[1] = ((addr & 0xFF000000) >>24 );
	CMD[2] = ((addr & 0x00FF0000) >>16 );
	CMD[3] = ((addr & 0x0000FF00) >>8 );

	MMC_Read_Block(CMD,Buffer,512);

	return(0);
}

//############################################################################
//Routine zum lesen des CID Registers von der MMC/SD-Karte (16Bytes)
unsigned char Read_CID_MMC (unsigned char *Buffer)
//############################################################################
{
	//Commando zum lesen des CID Registers
	unsigned char CMD[] = {0x4A,0x00,0x00,0x00,0x00,0xFF}; 
	
	MMC_Read_Block(CMD,Buffer,16);

	return(0);
}

//############################################################################
//Routine zum lesen des CSD Registers von der MMC/SD-Karte (16Bytes)
unsigned char Read_CSD_MMC (unsigned char *Buffer)
//############################################################################
{	
	//Commando zum lesen des CSD Registers
	unsigned char CMD[] = {0x49,0x00,0x00,0x00,0x00,0xFF};
	
	MMC_Read_Block(CMD,Buffer,16);

	return(0);
}


int MMC_GetCI() {
	return MMC_CI;
}

int MMC_GetWP() {
	return MMC_WP;
}
