/** \file
 * 
 * tcp.c
 *
 * Implementierung des TCP Protokolls.
 *
 * @version $Id: tcp.c,v 1.3 2005/08/08 10:11:03 fritz Exp $
 * @author oalt
 */

#include "tcp.h"
#include "buffers.h"
#include "ip.h"
#include "config.h"

extern TCPCON con;
extern byte send[MTU];

/**
 * Funktion behandelt eingehende TCP Pakete und fuehrt je nach Zustand des TCP Protokollautomaten
 * die entsprechenden Ma nahmen durch.
 * Der TCP Protokollautomat ist h i e r implementiert.
 */
void tcpmanager(byte *rcvdata, word datalen)  
{

  TCPHEADER * rtcph;
  byte flag;
  byte * data;
  
  rtcph=(TCPHEADER *)&(rcvdata[((rcvdata[0]&0x0F)<<2)]); // align TCPHEADER
 
  #ifdef DEBUG_ENABLE_MOST
    printfUART(UART_DEBUG,"Verbindungsstatus: %i\r\n",con.tcpstate);
  #endif
    
  datalen -= (rtcph->tcp_Len>>2);
  
  #ifdef DEBUG_ENABLE_MOST
    printfUART(UART_DEBUG,"Datenlaenge: %i\r\n",datalen);
  #endif
  
  data=(byte *)rtcph;

  data+=(rtcph->tcp_Len>>2);
  
  flag=rtcph->tcp_flag;
  
	#ifdef DEBUG_ENABLE_MOST
		printfUART(UART_DEBUG,"tcpflag: 0x%x\r\n",flag);
	#endif
    
  if(rtcph->tcp_dport_h==0 && rtcph->tcp_dport_l==80)  // http-request  
  {

	
    con.acknr=(dword) ((dword)rtcph->tcp_nnr_3<<24 |
                       (dword)rtcph->tcp_nnr_2<<16 |
                       (dword)rtcph->tcp_nnr_1<<8  |
                       (dword)rtcph->tcp_nnr_0);

    con.lport_h=0;
    con.lport_l=80;
    switch(con.tcpstate)
    {
     case LISTEN:
      if(flag==0x02) // connection request
      {
        con.seqnr=1;
        con.acknr++;
        con.rport_h=rtcph->tcp_sport_h;
        con.rport_l=rtcph->tcp_sport_l;

		#ifdef DEBUG_ENABLE_MOST
			printfUART(UART_DEBUG,"Connection request\r\n");
		#endif
        // con.tcpstate=SYN_RCVD;
        tcpsend(rcvdata,0,0x12,&con);  // send SYN + ACK
		con.tcpstate=SYN_RCVD;
        // return;
      } 
      break;
      
    case SYN_RCVD:
    {
      if(flag==0x10 || flag==0x18)
      {
      	#ifdef DEBUG_ENABLE_MOST
        printfUART(UART_DEBUG,"Verbindung etabliert !\r\n");
        #endif
        
        con.tcpstate=ESTABLISHED;
        // tcpsend(0,0x10,&con);
        
        // return;
      }
    } 
    break;
    case ESTABLISHED:
    
        if(datalen!=0)
        {
          con.acknr+=datalen;
          httpserver(data, datalen);
          
				#ifdef DEBUG_ENABLE_MOST
				printfUART(UART_DEBUG,"httpserver exit\r\n");
				#endif
	        }
    break;
    case CLOSE_WAIT:
      if(flag==0x11)
      {
        con.acknr++;
        tcpsend(rcvdata,0,0x10,&con);
        con.tcpstate=LISTEN;
      }
     break;
   } 
   
  } // http request
  

}

/** 
 * Prepare TCPHEADER for sending
 * Fuelle der TCP Header fuer das senden eines TCP Paketes und uebergebe ihn an die 
 * daruterliegende IP Schicht ( ipsend() )
 */
void tcpsend(byte *rcvdata, word datalen, byte flags, TCPCON * con)  
{

  // word len;
  TCPHEADER * out;
  dword ack=0;
  IPHDATA iphd;
  
    
  ack = con->acknr;
  // ack++;
  
  out=(TCPHEADER *)&send[100];
  
  out->tcp_sport_h=con->lport_h;
  out->tcp_sport_l=con->lport_l;

  out->tcp_dport_h=con->rport_h;
  out->tcp_dport_l=con->rport_l;
  
  out->tcp_acknr_3= (byte)(ack>>24 & 0xFF);
  out->tcp_acknr_2= (byte)(ack>>16 & 0xFF);
  out->tcp_acknr_1= (byte)(ack>>8  & 0xFF);
  out->tcp_acknr_0= (byte)(ack     & 0xFF);
 
  out->tcp_nnr_3= (byte)(con->seqnr>>24 & 0xFF);
  out->tcp_nnr_2= (byte)(con->seqnr>>16 & 0xFF);
  out->tcp_nnr_1= (byte)(con->seqnr>>8  & 0xFF);
  out->tcp_nnr_0= (byte)(con->seqnr     & 0xFF);

  con->seqnr+=(datalen + 1);

  out->tcp_Len=0x50;
  
  out->tcp_flag=flags; 
  
  out->tcp_slwind_h=0x02;
  out->tcp_slwind_l=0x40;
  
  out->tcp_urgptr_h=0;
  out->tcp_urgptr_l=0;
  
  out->tcp_chksum_h=0;
  out->tcp_chksum_l=0;
  
  printfUART(UART_DEBUG,"calculation checksum\r\n");
  tcpchksum(out,rcvdata,datalen);

  iphd.ip_protocol=6;
  iphd.ip_daddr_3=con->ipaddr_3;
  iphd.ip_daddr_2=con->ipaddr_2;
  iphd.ip_daddr_1=con->ipaddr_1;
  iphd.ip_daddr_0=con->ipaddr_0;

  printfUART(UART_DEBUG,"calling ipsend checksum\r\n");
  ipsend(datalen+20, iphd);  

}


/**
 * Calculate TCP Checksum
 *
 */
void tcpchksum(TCPHEADER * out, byte *rcvdata,word datalen)  
{

  word len=0, answer=0;
  dword sum=0;
  int cnt;
  
/*  sum= (word) (ip_read->data[12]<<8 | ip_read->data[13]);
  sum+=(word) (ip_read->data[14]<<8 | ip_read->data[15]);
  sum+=(word) (ip_read->data[16]<<8 | ip_read->data[17]);
  sum+=(word) (ip_read->data[18]<<8 | ip_read->data[19]);*/
  sum= (word) (rcvdata[12]<<8 | rcvdata[13]);
  sum+=(word) (rcvdata[14]<<8 | rcvdata[15]);
  sum+=(word) (rcvdata[16]<<8 | rcvdata[17]);
  sum+=(word) (rcvdata[18]<<8 | rcvdata[19]);


  sum+=(0x0006);
  
  len=datalen + (word) (out->tcp_Len>>2);
  
  sum+=len;
    
  for(cnt=0; cnt<len; cnt+=2)
  {
    sum+=((dword) ((word) (send[100+cnt]<<8) | (word)send[101+cnt])); 
  } 
  
  sum=(sum>>16) + (sum & 0xFFFF);
  sum+=(sum>>16);
  answer=~sum;

  out->tcp_chksum_h=(byte)((answer>>8) & 0xFF);
  out->tcp_chksum_l=(byte)(answer & 0x00FF);
  
}
