/** \file
 *
 * tpuart.c
 *
 * Hardware independend part for handling TPUART
 *
 * @version $Id: tpuart.c,v 1.8 2005/10/05 16:16:14 fritz Exp $
 * @author oalt
 *
 */

#include "config.h"
#include "globtype.h"
#include "uartapi.h"
#include "eib.h"
#include "tpuart.h"


byte rb[20][25]={0};		// receive queue
int tvals[25]={0};
int row_read=0;
int col_read=0;
int row_write=0;
int col_write=0;

byte sending = 0;
byte sendack = 0;
/**
 * Make TPUART working
 * @param port UART number (0..X)
 * @param mode TPUART mode (0: receive in busmonitor mode, 1: send)
 */
void initTPUART(int port, int mode)  {

  initUART(port, 19200, 8, 2, 1);
  initUARTir(port);
  initTimer();
  
  if(mode==0)  // receive
  {
    sendUARTbyte(port, u_activatebusmon);
    initUARTir(port);
    initTimer();
  }
}


/**
 * reset TP UART by software
 * @param port UART number (0..X)
 **/
void resetTPUARTsw(int port) {
	byte temp=0xFF;

	SSR2_RIE = 0; // disable receive interrupt
	sendUARTbyte(port,u_request_reset);
	while (temp==0xFF) {
		temp=getUARTbyte(port);
	}
	#ifdef DEBUG_ENABLE_MOST
	printfUART(UART_DEBUG,"received 0x%x after sw reset\r\n",temp);
	#endif
	SSR2_RIE = 1; // disable receive interrupt
}

/**
 * send an EIB frame via TPUART
 * 
 * @param port the UART port
 * @param ef the EIB frame to send
 * @return error code (not yet implemented)
 *
 */
int sendEIBframe(int port, EIBFRAME* ef)  {
  
	byte length, count=0;
	byte* eibframe;

	length=getFrameLen(ef);

	eibframe=(byte*)ef;
  
	#ifdef DEBUG_ENABLE_ALL
		printfUART(UART_DEBUG,"sending eibframe: 0x");
	#endif

	for(count=0; count<length-1; count++) {
		sendUARTbyte(port,(u_l_data | (byte)count));

		#ifdef DEBUG_ENABLE_ALL
			sendUARThex(UART_DEBUG,(u_l_data | (byte)count));
		#endif

		sending = 1;
		sendUARTbyte(port,eibframe[count]);

		#ifdef DEBUG_ENABLE_ALL
			sendUARThex(UART_DEBUG,eibframe[count]);
		#endif

		sending = 0; 
		if(sendack==1) {
			sendUARTbyte(port, 0x11);

			#ifdef DEBUG_ENABLE_ALL
				sendUARThex(UART_DEBUG,0x11);
			#endif

			sendack = 0;
		}
	}
	
	sendUARTbyte(port,(0x40 | length-1)); 

	#ifdef DEBUG_ENABLE_ALL
		sendUARThex(UART_DEBUG,(0x40 | length-1));
	#endif

	sending = 1;
	sendUARTbyte(port,eibframe[length-1]);

	#ifdef DEBUG_ENABLE_ALL
		sendUARThex(UART_DEBUG,eibframe[length-1]);
	#endif

	sending = 0;
	if(sendack==1) {
		sendUARTbyte(port, 0x11);

		#ifdef DEBUG_ENABLE_ALL
			sendUARThex(UART_DEBUG,0x11);
		#endif

		sendack = 0;
	}

	#ifdef DEBUG_ENABLE_ALL
		printfUART(UART_DEBUG,"\r\n");
	#endif

	return 0;
}

/**
 * Monitoring the received telegrams (hex code)
 * This function is for debugging only and not used in the gateway firmware.
 */

void rcmonitor(void)  {
  
  int i=0;
  byte data[3];
  // byte testframe[40];
  int datasize=3;

  data[0]=0x81;
  data[1]=0xFB;
  data[2]=0x85;

  // initEIBframe(&testframe[0],3, 0x1102, 0x0800, 1, 0x00, data, datasize);
  
  
  // sendEIBframe(&testframe[0]);
   
  while(1)
  {       
    while(row_write==row_read);
    
    for(i=0; i<((rb[row_read][5]&0x0F)+10); i++)
    {
      sendUARThex(1,rb[row_read][i]);
      rb[row_read][i]=0;
      sendUARTbyte(1,' ');
    }
    sendUARTbyte(1,'\r');
    row_read++;
    if(row_read==25)
    {
      row_read=0;
    }      
  }
}

/**
 * Trace the received telegrams (text). This routine never returns !
 * Debugging only.
 */

void telegramtrace(void)  {

  int i=0;
  byte len, priority, repeat, tpci, rc, daf, apcih, apcib;
  byte data[16];
  byte tmp;  
  byte abid=0; // apcib is data
  
  apcih=0;
  apcib=0;
    
  while(1)
  {
    printfUART(UART_DEBUG,"\r");
    while(row_write==row_read);
    
    len=(rb[row_read][5] & 0x0F)+7;
    priority=(rb[row_read][0] & 0x0C)>>2;
    repeat=(rb[row_read][0] & 0x20)>>5;
    daf=(rb[row_read][5] & 0x80)>>7; // 0: physical, 1: group adress    
    rc=(rb[row_read][5] & 0x70)>>4;
    
    if((rb[row_read][6] & 0x80)>>7)
    {
      tpci=rb[row_read][6];
    }
    else
    {
      tpci=rb[row_read][6] & 0xFC;
    }
    tmp=0;    
    tmp=((rb[row_read][6] & 0x03)<<2);
    apcih=tmp | ((rb[row_read][7] & 0xC0)>>6);
    apcib=(rb[row_read][7] & 0x3F);
    for(i=8; i<len; i++)
    {
      data[i-8]=rb[row_read][i]; // Octet 8 to n in data[]
    } 

    for(i=0; i<((rb[row_read][5]&0x0F)+9); i++)
    {
      sendUARThex(UART_DEBUG,rb[row_read][i]);
      sendUARTbyte(UART_DEBUG,' ');
    }
    sendUARTbyte(UART_DEBUG,'\r');
    
    printfUART(UART_DEBUG,"Analysis:\r");
    
    printfUART(UART_DEBUG,"Priority :");
    if(priority==0)
    {
      printfUART(UART_DEBUG," System\r");
    }
    if(priority==2)
    {
      printfUART(UART_DEBUG," Alarm\r");
    }
    if(priority==1)
    {
      printfUART(UART_DEBUG," High\r");
    }
    if(priority==3)
    {
      printfUART(UART_DEBUG," Low\r");
    }
    
    printfUART(UART_DEBUG,"Repeated frame:");
    if(repeat)
    {
      printfUART(UART_DEBUG," No\r");
    }
    else
    {
      printfUART(UART_DEBUG," Yes\r");
    }
 
    printfUART(UART_DEBUG,"Source Adress: ");
    sendUARThex(UART_DEBUG,rb[row_read][1]>>4);
    sendUARTbyte(UART_DEBUG,'.');
    sendUARThex(UART_DEBUG,rb[row_read][1]&&0x0F);
    sendUARTbyte(UART_DEBUG,'.');
    sendUARThex(UART_DEBUG,rb[row_read][2]);
    printfUART(UART_DEBUG,"\r");    
    
    printfUART(UART_DEBUG,"Dest. Adress:  ");
    // sendUARThex(IO,rb[row_read][3]);    
    // sendUARThex(IO,rb[row_read][4]);
    if(daf)
    {
      sendUARThex(UART_DEBUG,(rb[row_read][3]>>3) & 0x07);
      sendUARTbyte(UART_DEBUG,'/');
      sendUARThex(UART_DEBUG,rb[row_read][3] & 0x07);
      sendUARTbyte(UART_DEBUG,'/');    
      sendUARThex(UART_DEBUG,rb[row_read][4]);
      
      printfUART(UART_DEBUG," (Group Adress)\r");
    }
    else
    {
      sendUARThex(UART_DEBUG,rb[row_read][3]>>4);
      sendUARTbyte(UART_DEBUG,'.');
      sendUARThex(UART_DEBUG,rb[row_read][3]&&0x0F);
      sendUARTbyte(UART_DEBUG,'.');    
      sendUARThex(UART_DEBUG,rb[row_read][4]);
      printfUART(UART_DEBUG," (Physical Adress)\r");
    }
    printfUART(UART_DEBUG,"Routing Counter: ");
    sendUARThex(UART_DEBUG,rc);

    printfUART(UART_DEBUG,"\rTPCI-Type: ");
    if(tpci==0)
    {
      if(daf==1 && rb[row_read][3]==0 && rb[row_read][4]==0)
      {
        printfUART(UART_DEBUG,"T_BRODCAST_DATA_REQ_PDU\r");
      }
      else if(daf==1 && (rb[row_read][3]!=0 || rb[row_read][4]!=0))
      {
        printfUART(UART_DEBUG,"T_GROUPDATA_REQ_PDU\r");
      }      
      else
      {
        printfUART(UART_DEBUG,"T_DATA_UNACK_REQ_PDU\r");
      }
    }
    tmp=tpci>>6;
    switch(tmp)
    {
      case 1:
       printfUART(UART_DEBUG,"T_DATA_REQ_PDU ,seq. no.: 0x");
       sendUARThex(UART_DEBUG,(tpci & 0x3C)>>2);
       printfUART(UART_DEBUG,"\r");
      break;
      
      case 2:
        switch(tpci)
        {
          case 0x80:
           printfUART(UART_DEBUG,"T_CONNECT_REQ_PDU\r");
          break;
          case 0x81:
           printfUART(UART_DEBUG,"T_DISCONNECT_REQ_PDU\r");
          break;
          case 0x82:
            printfUART(UART_DEBUG,"T_CONNECT_CONF_PDU\r");
          break;
        }
        break;
      case 3:
        if((tpci & 0x03)==2)
        {
          printfUART(UART_DEBUG,"T_DATA_ACK_PDU, seq. no.: 0x");
        }
        else
        {
          printfUART(UART_DEBUG,"T_DATA_UNACK_PDU, seq. no.: 0x");
        }
        sendUARThex(UART_DEBUG,(tpci & 0x3C)>>2);
        printfUART(UART_DEBUG,"\r");   
      break;
    }
    
    if(len-7>0)
    {
    printfUART(UART_DEBUG,"APCI Type: ");
    
    switch(apcih)
    {
      case 0x00:
       printfUART(UART_DEBUG,"A_READ_VALUE_REQ_PDU\r");
      break;
      case 0x01:
       printfUART(UART_DEBUG,"A_READ_VALUE_RES_PDU\r");
       abid=1;
      break;
      case 0x02:
       printfUART(UART_DEBUG,"A_WRITE_VALUE_REQ_PDU\r");
       abid=1;
      break;
      case 0x03:
       printfUART(UART_DEBUG,"A_SET_PHYSADR_REQ_PDU\r");
      break;
      case 0x04:
       printfUART(UART_DEBUG,"A_READ_PHYSADR_REQ_PDU\r");
      break;
      case 0x05:
        printfUART(UART_DEBUG,"A_READ_PHYSADR_RES_PDU\r");
      break;
      case 0x06:
        printfUART(UART_DEBUG,"A_READ_ADC_REQ_PDU\r");
        abid=1;
      break;
      case 0x07:
        printfUART(UART_DEBUG,"A_READ_ADC_RES_PDU\r");
        abid=1;
      break;
      case 0x08:
        printfUART(UART_DEBUG,"A_READ_MEMORY_REQ_PDU\r");
      break;
      case 0x09:
        printfUART(UART_DEBUG,"A_READ_MEMORY_RES_PDU\r");
      break;
      case 0x0A:
        printfUART(UART_DEBUG,"A_WRITE_MEMORY_REQ_PDU\r");
      break;
      case 0x0B:
        switch(apcib)
        {
          case 0x00:
            printfUART(UART_DEBUG,"A_UREAD_MEMORY_REQ_PDU\r");
          break;
          case 0x01:
            printfUART(UART_DEBUG,"A_UREAD_MEMORY_RES_PDU\r");
          break;          
          case 0x02:
            printfUART(UART_DEBUG,"A_UWRITE_MEMORY_REQ_PDU\r");
          break;        
          case 0x04:
            printfUART(UART_DEBUG,"A_UWRITE_MEMORY_BIT_REQ_PDU\r");
          break;        
          case 0x05:
            printfUART(UART_DEBUG,"A_UREAD_MFACTINFO_REQ_PDU\r");
          break;        
          case 0x06:
            printfUART(UART_DEBUG,"A_UREAD_MFACTINFO_RES_PDU\r");
          break;        
          
        }
        if(apcib>=0x38 && apcib<=0x3E)
        {
          printfUART(UART_DEBUG,"manufacturer specific APCI\r");
        }
      break;
      case 0x0C:
        printfUART(UART_DEBUG,"A_READ_MASK_VERSION_REQ_PDU\r");
      break;
      case 0x0D:
        printfUART(UART_DEBUG,"A_READ_MASK_VERSION_RES_PDU\r");
      break;
      case 0x0E:
        printfUART(UART_DEBUG,"A_RESTART_REQ_PDU\r");
      break;
      case 0x0F:
        switch(apcib)
        {
          case 0x10:
            printfUART(UART_DEBUG,"A_WRITE_MEMORY_BIT_REQ_PDU\r");
          break;
          case 0x11:
            printfUART(UART_DEBUG,"A_AUTHORIZE_REQ_PDU\r");
          break;
          case 0x12:
            printfUART(UART_DEBUG,"A_AUTHORIZE_RES_PDU\r");
          break;
          case 0x13:
            printfUART(UART_DEBUG,"A_SETKEY_REQ_PDU\r");
          break;
          case 0x14:
            printfUART(UART_DEBUG,"A_SETKEY_RES_PDU\r");
          break;
          case 0x15:
            printfUART(UART_DEBUG,"A_READ_PROPRETY_VALUE_REQ_PDU\r");
          break;
          case 0x16:
            printfUART(UART_DEBUG,"A_READ_PROPRETY_VALUE_RES_PDU\r");
          break;
          case 0x17:
            printfUART(UART_DEBUG,"A_WRITE_PROPRETY_VALUE_REQ_PDU\r");
          break;
          case 0x18:
            printfUART(UART_DEBUG,"A_READ_PROPRETY_DESCRIPTION_REQ_PDU\r");
          break;
          case 0x19:
            printfUART(UART_DEBUG,"A_READ_PROPRETY_DESCRIPTION_RES_PDU\r");
          break;
          case 0x1C:
            printfUART(UART_DEBUG,"A_READ_PHYSADDR_SERNO_REQ_PDU\r");
          break;
          case 0x1D:
            printfUART(UART_DEBUG,"A_READ_PHYSADDR_SERNO_RES_PDU\r");
          break;
          case 0x1E:
            printfUART(UART_DEBUG,"A_SET_PHYSADDR_SERNO_REQ_PDU\r");
          break;
          case 0x1F:
            printfUART(UART_DEBUG,"A_SERVICE_INFORMATION_REQ_PDU\r");
          break;
          case 0x20:
            printfUART(UART_DEBUG,"A_SET_SYS_ID_REQ_PDU\r");
          break;
          case 0x21:
            printfUART(UART_DEBUG,"A_READ_SYS_ID_REQ_PDU\r");
          break;
          case 0x22:
            printfUART(UART_DEBUG,"A_READ_SYS_ID_RES_PDU\r");
          break;
          case 0x23:
            printfUART(UART_DEBUG,"A_READ_SYS_ID_SELECTIVE_REQ_PDU\r");
          break;
          case 0x3F:
            printfUART(UART_DEBUG,"Escape for EIBnet\r");
          break;
        
        }
      break;
    }
    }
    if(len>8 || abid)
    {
      printfUART(UART_DEBUG,"Data : ");
    }
    if(abid)
    {
      sendUARThex(UART_DEBUG,apcib);
      sendUARTbyte(UART_DEBUG,' ');
    }
    for(i=8; i<len; i++)
    {
      sendUARThex(UART_DEBUG,rb[row_read][i]);
      sendUARTbyte(UART_DEBUG,' ');
    }
        
    sendUARTbyte(UART_DEBUG,'\r');
    row_read++;
    if(row_read==25)
    {
      row_read=0;
    }    

  }

}

