/** \file
 *
 * uartapi.h
 *
 * This file includes the hardware independend functions for reading and writing to UART's
 * You need the hardware dependend functions sendUARTbyte(), getUARTbyte() and initUART()
 * for correct working. These functions are specified in the uart[chipnumber].c files.
 * <b>Before using these functions, call the initUART() function !!!</b>  
 *
 * @version $Id: uartapi.c,v 1.3 2005/08/04 13:58:53 fritz Exp $
 * @author oalt
 */

#include "globtype.h"
#include "uartapi.h"
#include <stdarg.h>


/*
 * write a string to UART
 * @param port port number of UART you want to use (0, 1, 2,...)
 * @param Name2 the string pointer (char*)
 * @return None
 */
 
void sendUARTstring(int port, char * Name2)	/* Puts a String to UART */
{
	int i=0;

	while (Name2[i]!=0x00) {
		sendUARTbyte(port,Name2[i++]);
	}
}

/**
 * Send Hex Number to UART (1 Byte)
 * @param port number of UART you want to use (0, 1, 2,...)
 * @param b the byte you want to send
 * @return None
 */

void sendUARThex(int port, byte b)  {

  byte tmp;
  
  tmp=(b & 0xF0)>>4;
  
  if(tmp<=9)
  {
    sendUARTbyte(port, 48+tmp);
  }
  else
  {
    sendUARTbyte(port, 65+tmp-10);
  }
  
  tmp=b & 0x0F;

  if(tmp<=9)
  {
    sendUARTbyte(port, 48+tmp);
  }
  else
  {
    sendUARTbyte(port, 65+tmp-10);
  }  
  
}

/**
 *
 * Send Hex Number to UART (integer)
 * @param port number of UART you want to use (0, 1, 2,...)
 * @param i the integer you want to send
 * @return None 
 */

void sendUARTihex(int port, int i)  {

  int tmp;
  
  tmp=(i & 0xFF00)>>8;
  sendUARThex(port,(byte)tmp);
  sendUARThex(port,(byte)i & 0x00FF);
  
}

/*
 * Send a number to UART
 * @param port number of UART you want to use (0, 1, 2,...)
 * @param number the number you want to send
 * @param base the base for the number (2, 10, ...)
 * @return None
 */

void sendUARTnumber(int port, int number, int base)  {				     
  int exp;
  int modulo;
  int teiler;
  int c;
  int t;
	
  teiler=1;
  exp=0;
  if(number<0)				// value negative ?
  {
	 sendUARTbyte(port,'-');
	 number*=(-1);
  }
	
  if(number==0)
  {
    sendUARTbyte(port,48);
  }
	
  while(number>=teiler) 
  {
	 exp++;
	 teiler=teiler*base;
  }
  modulo=number;
  
  for(c=0; c<=exp; c++) {
	  t=modulo/teiler;
      if(c!=0)
	  { 
	    	if(t<=10)
	    	{
	      	  sendUARTbyte(port,(byte)48+t);
	    	}
	    	else
	    	{
              sendUARTbyte(port,55+t); break;	
            }
      }
      modulo=modulo%teiler;
	  teiler=teiler/base;
	}    
}

/**
 * read a string from UART (echo included)
 * @param port number of UART you want to use (0, 1, 2,...)
 * @param str pointer of the string buffer (char *)
 * @return None
 */

void getUARTstring(int port, char * str)  {
  
  byte c;
  int i=0;
  
  do		
  {
	 c = getUARTbyte(port);  
	 if(c!=13)
	 {
	   sendUARTbyte(port,c);    // echo
	 }
	 *(str + i)=c;
	 i++;
  }while(c!=13);
   
  *(str + (i-1))='\0';    // terminate String
  
 
}

/**
 * print data to UART
 * usage like printf();
 * to be completed !!!!
 * @param port port number of UART you want to use (0, 1, 2,...)
 * @param fstring string to write 
 * @param ... optional values, data
 * @return None
 */

void printfUART(int port, char * fstring, ...)  {

  va_list pz;                     // Argumentzeiger 
  byte b;
  int i;
  char * string;

  pz=(char*)__builtin_va_alist;

  va_start(pz, fstring);          

  while (*fstring)                /* solange nicht das Nullzeichen */
  {
    if (*fstring != '%')          /* falls kein Formatelement */
     sendUARTbyte(port,*fstring);           /* Zeichen ausgeben */
    else                          /* sonst: */
    {
      switch(*++fstring)          /* n chstes Zeichen des Formatstrings */
      {
        case 'i':
         i = va_arg(pz, int);
         sendUARTnumber(port,i,10);
        break;
        
        case 'c':
         b = va_arg(pz, byte);
         sendUARTbyte(port,b);
        break;
        
        case 'x':
         b = va_arg(pz, byte);
         sendUARThex(port,b);
        break;

        case 'd':
         i = va_arg(pz, int);
         sendUARTihex(port,i);
        break;
                
        case 's':
         string = va_arg(pz,char*);
         sendUARTstring(port,string);
        break;
        
        case '%':
         sendUARTbyte(port,'%');
        break;
       /*  
        case 'S':
         s = va_arg(pz, short);   // va_arg liefert short-Wert 
         for (pos = 15; pos >= 0; pos--) // bin r ausgeben 
         {
           sendUARTnumber(CONSOLE,(s >> pos) & 1,10);
           if (pos % 8 == 0)      // Leerzeichen nach 8 Bit 
           {
             sendUARTbyte(CONSOLE,' ');
           }
          }
        break;

        case 'L':
         l = va_arg(pz, long);    // va_arg liefert long-Wert 
         for (pos = 31; pos >= 0; pos--) // bin r ausgeben 
         {
          // printf("%d", (l >> pos) & 1);
           if (pos % 8 == 0){}      // Leerzeichen nach 8 Bit 
            //  printf(" ");
          }
        break;
        */
        default:
         sendUARTbyte(port,*fstring);      /* kein Formatsymbol nach %:
                                     Zeichen ausgeben */
       }                          /* Ende switch */
     }                           /* Ende else */
     fstring++;                   /* auf n chstes Zeichen im Formatstring */
  }                              /* Ende while */
  va_end(pz);                      /* Auswertungsende: NULL nach pz */

}

